@@ -19,9 +19,9 @@
 */
 
 
-#define GTK_DISABLE_DEPRECATED 1
-#define GDK_DISABLE_DEPRECATED 1
-#define G_DISABLE_DEPRECATED 1
+// #define GTK_DISABLE_DEPRECATED 1
+// #define GDK_DISABLE_DEPRECATED 1
+// #define G_DISABLE_DEPRECATED 1
 #define _XOPEN_SOURCE 700 // SUSv4
 #include <config.h>
 
@@ -33,7 +33,6 @@
 #include <errno.h>
 #include <fcntl.h> // open()
 #include <algorithm>
-
 G_BEGIN_DECLS
 #include "callbacks.h"
 #include "interface.h"
@@ -81,7 +80,7 @@ G_MODULE_EXPORT void
 on_file_exit_activate                       (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-  exit(0);
+  exit( 0 );
 }
 
 
@@ -92,7 +91,6 @@ on_help_about_activate                      (GtkMenuItem     *menuitem,
 {
   GtkWidget* about = create_about_dialog();
   gtk_dialog_run(GTK_DIALOG(about));
-  // gtk_widget_destroy(about);
   gtk_widget_hide(about);
 }
 
@@ -107,8 +105,7 @@ on_window_delete_event                 (GtkWidget       *widget,
 
 
 G_MODULE_EXPORT void
-on_window_destroy                      (GtkObject       *object,
-                                        gpointer         user_data)
+on_window_destroy( GtkWidget* widget, gpointer user_data )
 {
   exit(0);
 }
@@ -311,19 +308,13 @@ on_view_big_activate                        (GtkMenuItem     *menuitem,
 
 /** ウィンドウの露出。転送するだけ。 */
 G_MODULE_EXPORT gboolean
-on_drawingarea_expose_event( GtkWidget* widget,
-                             GdkEventExpose* event,
-                             gpointer user_data )
+on_drawingarea_draw( GtkWidget* widget, cairo_t* cr, gpointer user_data )
 {
-  gdk_draw_drawable( widget->window, 
-                     widget->style->fg_gc[ GTK_WIDGET_STATE(widget) ],
-                     daemon_canvas_get_pixmap(g_canvas), // src
-                     event->area.x,
-                     event->area.y,
-                     event->area.x,
-                     event->area.y,
-                     event->area.width,
-                     event->area.height );
+  cairo_set_source_surface(cr, daemon_canvas_get_pixmap(g_canvas), 0, 0);
+  int width = gtk_widget_get_allocated_width( widget );
+  int height = gtk_widget_get_allocated_height( widget );
+  cairo_rectangle( cr, 0, 0, width, height );
+  cairo_fill( cr );
 
   return FALSE;
 }
@@ -342,19 +333,6 @@ on_drawingarea_leave_notify_event      (GtkWidget       *widget,
 
 extern NetGameInfo game_info;
 
-/** gtk_combo_box_new_text() 相当のコード */
-static void combo_box_set_text_mode( GtkComboBox* w )
-{
-  GtkListStore* store = gtk_list_store_new(1, G_TYPE_STRING);
-  gtk_combo_box_set_model( w, GTK_TREE_MODEL(store) );
-  g_object_unref(store);
-
-  GtkCellRenderer* cell = gtk_cell_renderer_text_new();
-  gtk_cell_layout_pack_start( GTK_CELL_LAYOUT(w), cell, TRUE );
-  gtk_cell_layout_set_attributes( GTK_CELL_LAYOUT(w), cell, 
-				    "text", 0, NULL );
-}
-
 
 /** 
  * ゲーム開始ダイアログ.
@@ -369,21 +347,19 @@ on_dialog_game_realize             (GtkWidget       *widget,
   for (int i = 0; i < 2; i++) {
     w = lookup_widget(widget, i == 0 ? "optionmenu_sente" : "optionmenu_gote");
 
-    combo_box_set_text_mode( GTK_COMBO_BOX(w) );
-    gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("HUMAN"));
-    gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL1"));
-    gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL2"));
-    gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL3"));
-    gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("BONANZA"));
+    gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("HUMAN"));
+    gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL1"));
+    gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL2"));
+    gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL3"));
+    gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("BONANZA"));
 
     gtk_combo_box_set_active(GTK_COMBO_BOX(w), 0);
   }
 
   // 手合
   w = lookup_widget(widget, "optionmenu_teai");
-  combo_box_set_text_mode( GTK_COMBO_BOX(w) );
-  gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("HIRATE"));
-  gtk_combo_box_append_text (GTK_COMBO_BOX (w), _("Just current board"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("HIRATE"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("Just current board"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(w), 0);
 }
 
@@ -434,7 +410,7 @@ static gboolean on_connecting_idle(gpointer data)
   g_signal_connect_after(G_OBJECT(g_canvas->window), "notify::startgame",
 			 daemon_game_start_via_network, NULL);
   // ログインする
-  csa_login_and_wait_socket(worker.username.c_str(), worker.password.c_str());
+  csa_login_and_wait_socket( &worker );
 
   return FALSE; // アイドル待ちは終了
 }
@@ -448,8 +424,17 @@ extern void add_watch_socket();
  */
 void daemon_game_start_via_network()
 {
+  printf("%s enter.\n", __func__);
+
+  // 二重に呼び出されることがある
+  if (g_canvas->mode != D_CANVAS_MODE_PRE_GAME) {
+    printf("mode is %d. return.\n", g_canvas->mode);
+    return;
+  }
+
   PLAYERTYPE playertype[2];
 
+  // watchしなおす
   add_watch_socket();
 
   // TODO: 平手以外
@@ -458,16 +443,21 @@ void daemon_game_start_via_network()
   if (game_info.my_turn == '+') {
     playertype[0] = worker.player_type;
     playertype[1] = SI_NETWORK_1;
+
+    if (g_canvas->front != 0)
+      on_view_flip_activate(NULL, NULL);
   }
   else {
     playertype[0] = SI_NETWORK_1;
     playertype[1] = worker.player_type;
 
-    on_view_flip_activate(NULL, NULL);
+    if (g_canvas->front == 0)
+      on_view_flip_activate(NULL, NULL);
   }
 
   daemon_game_setup(playertype[0], game_info.player_name[0].c_str(),
-                    playertype[1], game_info.player_name[1].c_str());
+                    playertype[1], game_info.player_name[1].c_str(),
+		    game_info.total_time );
 
   // main loop
   daemon_game_main_loop();
@@ -477,7 +467,7 @@ void daemon_game_start_via_network()
 extern PLAYERTYPE get_player_type(int combo_index);
 // int network_game_start = 0;
 
-/** ゲーム -> CSAサーバに接続 */
+/** メニュー: ゲーム -> CSAサーバに接続 */
 G_MODULE_EXPORT void 
 on_game_remote_activate           (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -494,17 +484,19 @@ on_game_remote_activate           (GtkMenuItem     *menuitem,
   GtkEntry* server_port_entry = GTK_ENTRY(lookup_widget(dialog, "server_port"));
   GtkEntry* username_entry = GTK_ENTRY(lookup_widget(dialog, "username"));
   GtkEntry* password_entry = GTK_ENTRY(lookup_widget(dialog, "password"));
+  GtkCheckButton* accept_immediately = GTK_CHECK_BUTTON(
+				lookup_widget(dialog, "accept_immediately"));
   assert( server_host_entry && server_port_entry && username_entry && 
-	  password_entry );
+	  password_entry && accept_immediately );
 
   gtk_entry_set_text(server_host_entry, worker.hostname.c_str());
   gtk_entry_set_text(server_port_entry, worker.port.c_str());
   gtk_entry_set_text(username_entry, worker.username.c_str());
   gtk_entry_set_text(password_entry, worker.password.c_str());
-
+  gtk_toggle_button_set_active(
+           GTK_TOGGLE_BUTTON(accept_immediately), worker.accept_immediately);
   while (1) {
     if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
-      // gtk_widget_destroy(dialog);
       gtk_widget_hide(dialog);
       return;
     }
@@ -516,6 +508,8 @@ on_game_remote_activate           (GtkMenuItem     *menuitem,
     GtkWidget* w = lookup_widget(dialog, "playertype_combobox");
     worker.player_type = get_player_type(
 			     gtk_combo_box_get_active(GTK_COMBO_BOX(w)));
+    worker.accept_immediately = gtk_toggle_button_get_active(
+				     GTK_TOGGLE_BUTTON(accept_immediately));
 
     // TODO: 設定ファイルへの保存
 
@@ -538,7 +532,6 @@ on_game_remote_activate           (GtkMenuItem     *menuitem,
       break;
   }
 
-  // gtk_widget_destroy(dialog);
   gtk_widget_hide(dialog);
 
   if (!connect_status_dialog)
@@ -725,7 +718,8 @@ on_view_flip_activate(GtkMenuItem* menuitem, gpointer user_data)
 
   flip_menu = lookup_widget(canvas->window, "flip");
   assert(flip_menu);
-  GTK_CHECK_MENU_ITEM(flip_menu)->active = canvas->front != SENTE;
+  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(flip_menu),
+				  canvas->front != SENTE );
 
   // 再描画
   canvas_redraw(canvas);
@@ -779,7 +773,6 @@ static void daemon_game_mate_start( MateParam* param, int tesuki,
     break;
   }
 
-  // gtk_widget_destroy(dialog);
   gtk_widget_hide(dialog);
 
   if (r)
@@ -911,7 +904,6 @@ on_game_new_game_activate(GtkBin* menuitem, gpointer user_data)
   if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
     r = daemon_game_new_game_ok(dialog);
 
-  // gtk_widget_destroy(dialog);
   gtk_widget_hide(dialog);
 
   if (r)
@@ -1099,7 +1091,6 @@ static void on_alt_moves_comment_editing_started( GtkCellRenderer* renderer,
 			  te_count, n->moves[list_idx] );
   }
 
-  // gtk_widget_destroy(dialog);
   gtk_widget_hide(dialog);
   gtk_tree_path_free(path);
 
@@ -1258,8 +1249,7 @@ on_connecting_cancelbutton_activate    (GtkButton       *button,
 
 /** 主ウィンドウ */
 G_MODULE_EXPORT void
-on_window_realize                      (GtkWidget       *widget,
-                                        gpointer         user_data)
+on_window_realize( GtkWidget* widget, gpointer user_data )
 {
   canvas_build(g_canvas);
 }
@@ -1272,12 +1262,11 @@ on_remote_dialog_realize               (GtkWidget       *widget,
   GtkWidget* w = lookup_widget(widget, "playertype_combobox");
   assert(w);
 
-  combo_box_set_text_mode( GTK_COMBO_BOX(w) );
-  gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("HUMAN"));
-  gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL1"));
-  gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL2"));
-  gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("LEVEL3"));
-  gtk_combo_box_append_text( GTK_COMBO_BOX(w), _("BONANZA"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("HUMAN"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL1"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL2"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("LEVEL3"));
+  gtk_combo_box_text_append_text( GTK_COMBO_BOX_TEXT(w), _("BONANZA"));
   gtk_combo_box_set_active(GTK_COMBO_BOX(w), 0);
 }
 
@@ -1317,29 +1306,36 @@ on_game_solve_hisshi_activate          (GtkMenuItem     *menuitem,
 
 /** 変化手順の削除 */
 G_MODULE_EXPORT void
-on_alt_moves_delete_button_activate    (GtkButton       *button,
-                                        gpointer         user_data)
+on_alt_moves_delete_button_activate( GtkButton* button, gpointer user_data )
 {
   KyokumenNode* n = g_canvas->record->mi;
   int te_count;
   for( te_count = 0; te_count < g_canvas->history->cur_pos; te_count++ )
     n = n->lookup_child( g_canvas->history->focus_line[te_count].first );
 
-  // どれか
+  // どの指し手か
   GtkTreeView* next_moves = GTK_TREE_VIEW(
-  	  lookup_widget(g_canvas->move_property_dialog, "next_moves"));
+        	 lookup_widget(g_canvas->move_property_dialog, "next_moves"));
   assert( next_moves );
 
-  GtkTreePath* path = NULL;
-  gtk_tree_view_get_cursor( next_moves, &path, NULL );
-  assert( path );
-  int idx = *gtk_tree_path_get_indices( path );
+  // cursor ではなく, selection を使う
+  GtkTreeSelection* sel = gtk_tree_view_get_selection( next_moves );
+  if ( !sel )
+    return;
+  GList* rows = gtk_tree_selection_get_selected_rows( sel, NULL );
+  if ( !rows || !g_list_length(rows) ) {
+    g_list_free_full( rows, (GDestroyNotify) gtk_tree_path_free );
+    return;
+  }
 
   if ( daemon_canvas_yes_no_dialog(g_canvas, _("Are you sure?")) ==
-       GTK_RESPONSE_YES ) {
+                                                        GTK_RESPONSE_YES ) {
+    GtkTreeModel* model = gtk_tree_view_get_model( next_moves );
+    GtkTreePath* path = (GtkTreePath*) g_list_nth_data(rows, 0);
+    int idx = *gtk_tree_path_get_indices( path );
+
     n->remove_child( n->moves[idx].te );
 
-    GtkTreeModel* model = gtk_tree_view_get_model( next_moves );
     GtkTreeIter iter;
     gboolean r = gtk_tree_model_iter_nth_child( model, &iter, NULL, idx );
     assert( r );
@@ -1351,13 +1347,13 @@ on_alt_moves_delete_button_activate    (GtkButton       *button,
       g_canvas->history->sync( *g_canvas->record, g_canvas->history->cur_pos,
 			       n->moves[idx].te );
     }
-    else {
+    else 
       g_canvas->history->delete_move( te_count );
-    }
+
     g_canvas->history->highlight_cur_pos();
   }
 
-  gtk_tree_path_free( path );
+  g_list_free_full( rows, (GDestroyNotify) gtk_tree_path_free );
 }
 
 
