@@ -38,9 +38,7 @@ on_drawingarea_button_release_event    (GtkWidget       *widget,
                                         gpointer         user_data);
 
 gboolean
-on_drawingarea_expose_event            (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data);
+on_drawingarea_draw( GtkWidget* widget, cairo_t* cr, gpointer user_data );
 
 gboolean
 on_drawingarea_leave_notify_event      (GtkWidget       *widget,
@@ -166,8 +164,7 @@ on_window_delete_event                 (GtkWidget       *widget,
                                         GdkEvent        *event,
                                         gpointer         user_data);
 
-void
-on_window_destroy                      (GtkObject       *object,
+void on_window_destroy( GtkWidget* widget,
                                         gpointer         user_data);
 
 void
@@ -210,8 +207,7 @@ void
 on_connecting_cancelbutton_activate    (GtkButton       *button,
                                         gpointer         user_data);
 
-void
-on_window_realize                      (GtkWidget       *widget,
+void on_window_realize( GtkWidget* widget,
                                         gpointer         user_data);
 
 void
