@@ -57,16 +57,20 @@ CharInfo* dameon_charinfo_new_from_file(GtkWidget* window,
   return info;
 }
 
+
 /** info を初期化する */
-void daemon_charinfo_init(CharInfo* info) {
+void daemon_charinfo_init(CharInfo* info) 
+{
   memset(info, 0, sizeof(CharInfo));
 }
 
+
 /**
  * 勝敗の種類に併せて画像を返す。
  * 
  */
-GdkPixmap* daemon_charinfo_get_image(CharInfo* info, MATCH_STATE s) {
+GdkPixbuf* daemon_charinfo_get_image( CharInfo* info, MATCH_STATE s ) 
+{
   return info->pixmap;
 }
 
@@ -87,10 +91,12 @@ const gchar* daemon_charinfo_get_word(CharInfo* info, MATCH_STATE s) {
   return NULL;
 }
 
+
 /** デフォルトデータで CharInfo を生成した返す。 */
-CharInfo* daemon_charinfo_get_daemon(GtkWidget* window) {
+CharInfo* daemon_charinfo_get_daemon(GtkWidget* window) 
+{
   CharInfo* info;
-  GdkPixbuf* im;
+  // GdkPixbuf* im;
   gint w, h;
   
   info = daemon_charinfo_new(window);
@@ -98,13 +104,13 @@ CharInfo* daemon_charinfo_get_daemon(GtkWidget* window) {
   strncpy(info->char_name, "daemon", sizeof(info->char_name));
   
   /* 画像を読む */
-  im = gdk_pixbuf_new_from_xpm_data(daemon_xpm);
-  if (im == NULL) {
+  info->pixmap = gdk_pixbuf_new_from_xpm_data( daemon_xpm );
+  if ( !info->pixmap ) {
     printf("No enough memory in daemon_charinfo_get_daemon(). im is NULL\n");
     abort();
   }
-  gdk_pixbuf_render_pixmap_and_mask(im, &info->pixmap, NULL, 0);
-  g_object_unref(im);
+  // gdk_pixbuf_render_pixmap_and_mask(im, &info->pixmap, NULL, 0);
+  // g_object_unref(im);
 
   strncpy(info->words[0][0], "は、しまった！", sizeof(info->words[0][0]));
   strncpy(info->words[1][0], "勝負はこれからだよ！", sizeof(info->words[1][0]));
