@@ -55,18 +55,21 @@ typedef enum {
 
 
 /** キャラクターの情報をまとめて扱うクラス */
-typedef struct {
+struct CharInfo
+{
   /** キャラクターの名前 */
   gchar char_name[256];
+
   /** キャラクターの画像 */
-  GdkPixmap* pixmap;
+  GdkPixbuf* pixmap;
+
   /** キャラクターの台詞 */
   gchar words[6][4][256];
   /** 前回の台詞の番号 */
   gint last_word;
   /** キャラクターの強さ */
   gint level;
-} CharInfo;
+} ;
   
 
 /*********************************************************/
@@ -78,8 +81,10 @@ CharInfo*    dameon_charinfo_new_from_file(GtkWidget* window,
 					   gchar* image_file,
 					   gchar* words_file);
 void         daemon_charinfo_init         (CharInfo* info);
-GdkPixmap*   daemon_charinfo_get_image    (CharInfo* info,
+
+GdkPixbuf* daemon_charinfo_get_image( CharInfo* info,
 					   MATCH_STATE s);
+
 const gchar* daemon_charinfo_get_word     (CharInfo* info,
 					   MATCH_STATE s);
 CharInfo*    daemon_charinfo_get_daemon   (GtkWidget* window);
