@@ -25,9 +25,9 @@
 // http://msdn.microsoft.com/en-us/library/ms740673(VS.85).aspx
 
 
-#define GTK_DISABLE_DEPRECATED 1
-#define GDK_DISABLE_DEPRECATED 1
-#define G_DISABLE_DEPRECATED 1
+// #define GTK_DISABLE_DEPRECATED 1
+// #define GDK_DISABLE_DEPRECATED 1
+// #define G_DISABLE_DEPRECATED 1
 
 #include <config.h>
 
@@ -62,34 +62,21 @@ G_END_DECLS
 
 #include "conn.h"
 #include "canvas.h"
-#include "si/ui.h"
 
 
 /** 行に分けた受信データ */
-static RecvBuffer recv_buffer;
+static RecvBuffer* recv_buffer = NULL;
 
 static int wait_state = 0;
 static gint io_watch = 0;
 static int server_fd = -1;
-// static guint idle_id = 0;
 
-/** 受信した指し手 */
-struct NetTe 
-{
-  /** 指し手. 投了, 千日手なども */
-  DTe te;
-
-  /** 消費秒数 */
-  int sec;
-
-  NetTe(): sec(0) { }
-};
-
-typedef list<NetTe> NetTeList;
-static NetTeList net_te_list;
+typedef list<string> NetTeList;
+static NetTeList net_queue;
 
 extern GtkWidget* connect_status_dialog;
 
+
 /** サーバとの接続を切断。 */
 void disconnect_server()
 {
@@ -98,34 +85,20 @@ void disconnect_server()
   if (server_fd < 0)
     return;
 
-  recv_buffer.clear();
+  delete recv_buffer;
+  recv_buffer = NULL;
   wait_state = 0;
-  net_te_list.clear();
+  net_queue.clear();
 
   g_source_remove(io_watch);
   io_watch = 0;
 
   closesocket(server_fd);
   server_fd = -1;
-
-  // daemon_canvas_change_mode(g_canvas, D_CANVAS_MODE_BOOK);
 }
 
 
-/** 0終端の文字列を送信 */
-static void csa_send(const char* line)
-{
-  printf("send: %s", line); // DEBUG
-  int ret = write(server_fd, line, strlen(line));
-  if (ret < 0) {
-    printf("%s: socket write error.\n", __func__);
-    exit(1);
-  }
-}
-
-
-static string username;
-
+static const ConnectThreadArg* connect_arg;
 NetGameInfo game_info;
 
 
@@ -134,9 +107,8 @@ NetGameInfo game_info;
  */
 static bool parse_time()
 {
-  while ( recv_buffer.recv_list.size() > 0 ) {
-    string line = recv_buffer.recv_list.front();
-    recv_buffer.recv_list.pop_front();
+  while ( recv_buffer->recv_list.size() > 0 ) {
+    string line = recv_buffer->get_front();
 
     if (line.find("END Time") == 0) {
       wait_state = 2;
@@ -162,10 +134,10 @@ static bool parse_time()
     }
     else if (name == "Total_Time")
       game_info.total_time = atoi(value.c_str());
-    else if (name == "Byoyomi")
-      ;
-    else if (name == "Least_Time_Per_Move")
-      ;
+    else if (name == "Byoyomi") {
+    }
+    else if (name == "Least_Time_Per_Move") {
+    }
   }
 
   return false; // 引き続き呼び出してもらう
@@ -184,9 +156,8 @@ static bool state_2_read()
 
   bool finished = false;
 
-  while ( recv_buffer.recv_list.size() > 0 ) {
-    string line = recv_buffer.recv_list.front();
-    recv_buffer.recv_list.pop_front();
+  while ( recv_buffer->recv_list.size() > 0 ) {
+    string line = recv_buffer->get_front();
 
     string name, value;
     int pos;
@@ -235,10 +206,9 @@ static bool state_2_read()
 static bool state_3_read()
 {
   printf("%s: line = %s\n", __func__, 
-	 recv_buffer.recv_list.front().c_str()); // DEBUG
+	 recv_buffer->recv_list.front().c_str()); // DEBUG
 
-  string line = recv_buffer.recv_list.front(); 
-  recv_buffer.recv_list.pop_front();
+  string line = recv_buffer->get_front(); 
   if (line.find("START:", 0) == string::npos) {
     // reject ?
     // network_log_append_message("reject game.\n");
@@ -252,188 +222,17 @@ static bool state_3_read()
 }
 
 
-const char csa_koma_str[][3] = {
-  "",   "FU", "KY", "KE", "GI", "KI", "KA", "HI",
-  "OU", "TO", "NY", "NK", "NG", "",   "UM", "RY",
-};
-
-/** 手を作る. daemon_record_load_csa_te() を参考に。 */
-static void parse_te( const string& line, NetTe* net_te )
-{
-  assert( net_te );
-
-  int idx = 0;
-
-  if ( !line.find("%TORYO") ) {
-    net_te->te.clear();
-    net_te->te.special = DTe::RESIGN;
-    idx += strlen("%TORYO");
-  }
-  else if ( !line.find("%KACHI") ) {
-    net_te->te.clear();
-    net_te->te.special = DTe::KACHI;
-    idx += strlen("%KACHI");
-  }
-  else {
-    // 通常の指し手
-    assert( line[0] == '+' || line[0] == '-' );
-
-    char koma_str[3];
-    PieceKind p;
-
-    net_te->te.clear();
-    net_te->te.special = DTe::NORMAL_MOVE;
-    // xyが逆
-    Xy fm = ((line[2] - 0x30) << 4) + line[1] - 0x30;
-    Xy to = ((line[4] - 0x30) << 4) + line[3] - 0x30;
-
-    sscanf(line.c_str() + 5, "%2s", koma_str );
-    for (p = 1; p < 0xf; p++) {
-      if (strcmp(koma_str, csa_koma_str[p]) == 0)
-	break;
-    }
-
-    if ( fm == 0 ) {
-      // 駒打ち
-      net_te->te.to = to; net_te->te.uti = p;
-    }
-    else {
-      // 移動
-      PieceKind pf = daemon_dboard_get_board(&g_canvas->board, 
-				     fm & 0xf, 
-				     fm >> 4) & 0xf;
-      net_te->te.fm = fm;
-      net_te->te.to = to;
-      net_te->te.nari = pf <= 8 && p >= 9 ? true : false;
-      net_te->te.tori = g_canvas->board.board[to];
-    }
-
-    idx += 7;
-  }
-
-  // 消費時間
-  net_te->sec = 1; // 0 はない
-  sscanf( line.c_str() + idx, ",T%d", &net_te->sec );
-
-  printf("%s: parsed = %s, %d\n", 
-	 __func__, net_te->te.to_str(g_canvas->board).c_str(), net_te->sec);
-}
-
-
-static NetTe pending_move;
-
-/** 相手の手または自分の手番で時間切れを受信 */
+/** 
+ * 相手の手または自分の手番で時間切れを受信
+ * 先読みがあるので、ここでは読むだけ
+ */
 static bool state_4_read()
 {
   printf("%s: front = '%s'\n", __func__, 
-	 recv_buffer.recv_list.front().c_str()); // DEBUG
-
-  string line = recv_buffer.recv_list.front();
-  recv_buffer.recv_list.pop_front();
-
-  NetTe net_te;
-
-  if (line == "#JISHOGI") {
-    // プロトコルv1.1.2では「日本将棋連盟ルール」とだけ書いている.
-    // 24点法だと思うが、そうすると引き分けの場合がある
-    //
-    // http://www.computer-shogi.org/wcsc20/rule.html
-    // によれば, プロトコルを無視して, 27点法によるみたい
-    //   → 引き分けはない
-    assert( pending_move.te.special == DTe::KACHI );
-  }
-  else if (line == "#SENNICHITE") {
-    // 連続王手でない千日手. 指してもよいがそこで終了. これだけ扱いが違う
-    // クライアントでも千日手チェックするか, 非同期で受信しないといけない
-    //   +6978KI,T2
-    //   #SENNICHITE
-    //   #DRAW
-    assert( pending_move.te.special == DTe::SENNICHITE );
-  }
-  else if (line == "#OUTE_SENNICHITE") {
-    // 連続王手の千日手
-    // これもクライアントでのチェックが必要
-    //   -8493OU,T2
-    //   #OUTE_SENNICHITE
-    //   #WIN
-    assert( pending_move.te.special == DTe::ILLEGAL );
-  }
-  else if (line == "#ILLEGAL_MOVE") {
-    // これもクライアントでのチェックが必要
-    //   +0031FU,T1
-    //   #ILLEGAL_MOVE
-    //   #WIN
-    assert( pending_move.te.special == DTe::ILLEGAL || 
-	    pending_move.te.special == DTe::KACHI );
-
-    pending_move.te.special = DTe::ILLEGAL; // 不正な入玉勝利宣言
-  }
-  else if (line == "#TIME_UP") {
-    // 時間切れ. 1行目で判断できる
-    // #TIME_UP
-    // #WIN
-    pending_move.te.special = DTe::TIME_OUT;
-  }
-  else if (line == "#CHUDAN") {
-    // サーバからの中断指示
-    // クライアントからの中断要請もできるが, プロトコルv1.1.2では何が起こるか不明
-    gmSetGameStatus( &g_game, SI_CHUDAN_REMOTE );
-  }
-  else if ( line == "#RESIGN" ) {
-    assert( pending_move.te.special == DTe::RESIGN );
-  }
-  else if ( line == "#WIN" || line == "#LOSE" || line == "#DRAW" ) {
-    assert( pending_move.te.special );
-
-    if ( pending_move.te.special == DTe::TIME_OUT )
-      gmSetGameStatus( &g_game, SI_TIMEOUT );
-    else {
-      net_te_list.push_back( pending_move );
-      printf("%s: add net_te = %s\n", __func__, 
-	   pending_move.te.to_str(g_canvas->board).c_str() ); // DEBUG
-    }
-  }
-  else {
-    // 通常手, 投了, 勝利宣言
-    parse_te( line, &net_te );
-    bool forbidden = false;
-    const BOARD* bo = &g_canvas->board;
-
-    if (net_te.te.special == DTe::RESIGN) {
-      // 勝ったとき. #RESIGN, #WINは無視してよい
-      //   %TORYO.T4
-      //   #RESIGN
-      //   #WIN
-      pending_move = net_te;
-    }
-    else if (net_te.te.special == DTe::KACHI) {
-      // 勝利宣言. クライアントでチェックするか2行目を待つか
-      // shogi-server では #DRAW を返すことはないようだが?
-      //   %KACHI,T8
-      //   #JISHOGI
-      //   #WIN
-      // または
-      //   %KACHI,T8
-      //   #ILLEGAL_MOVE
-      //   #WIN
-      pending_move = net_te;
-    }
-    else if ( !te_is_valid(bo, net_te.te.pack()) ) {
-      pending_move = net_te;
-      pending_move.te.special = DTe::ILLEGAL;
-      // #ILLEGAL_MOVE を待つ
-    }
-    else if ( te_is_sennichite(bo, net_te.te.pack(), &forbidden) ) {
-      pending_move = net_te;
-      pending_move.te.special = forbidden ? DTe::ILLEGAL : DTe::SENNICHITE;
-      // #OUTE_SENNICHITE または #SENNICHITE を待つ
-    }
-    else {
-      net_te.te.special = DTe::NORMAL_MOVE;
-      net_te_list.push_back(net_te);
-    }
-  }
+	 recv_buffer->recv_list.front().c_str()); // DEBUG
 
+  string line = recv_buffer->get_front();
+  net_queue.push_back(line);
   return true;
 }
 
@@ -445,8 +244,10 @@ static bool state_4_read()
 INPUTSTATUS daemon_input_next_network_impl( const BOARD* bo, 
 					    DTe* te, int* elapsed_time )
 {
+  printf( "%s: net_queue.size() = %lu\n", __func__, net_queue.size() ); // DEBUG
+
   // 手が設定されるまで待つ
-  while (net_te_list.size() == 0) {
+  while (net_queue.size() == 0) {
     // 時間切れ, 中断
     if ( gmGetGameStatus(&g_game) != 0 )
       return gmGetGameStatus(&g_game);
@@ -454,28 +255,27 @@ INPUTSTATUS daemon_input_next_network_impl( const BOARD* bo,
     processing_pending( true );
   }
 
-  NetTe net_te = net_te_list.front();
-  net_te_list.pop_front();
-
-  *te = net_te.te;
-  *elapsed_time = net_te.sec;
-  return SI_NORMAL;
+  INPUTSTATUS r = conn_queue_front_move(net_queue, bo, te, elapsed_time);
+  if ( r == SI_CHUDAN_REMOTE || r == SI_TIMEOUT )
+    gmSetGameStatus(&g_game, r );
+  return r;
 }
 
 
 /** ログイン待ち */
 static void state_1_read()
 {
-  string s = "LOGIN:" + username + " OK";
-  if ( recv_buffer.recv_list.front() == s) {
+  string s = "LOGIN:" + connect_arg->username + " OK";
+  if ( recv_buffer->get_front() == s) {
     // network_log_append_message(_("%s login ok.\n"), arg->username.c_str());
     // arg->status_queue.push_back(_("login ok."));
-    recv_buffer.recv_list.pop_front();
     wait_state = 2;
   }
   else {
     disconnect_server();
     daemon_messagebox(g_canvas, _("login failed."), GTK_MESSAGE_ERROR);
+
+    // ダイアログ再表示
     g_signal_emit_by_name(
 		  G_OBJECT(lookup_widget(g_canvas->window, "connect_server")),
 		  "activate", NULL);
@@ -483,16 +283,6 @@ static void state_1_read()
 }
 
 
-#if 0
-struct SocketHook {
-  bool (*on_game_proposal)();
-  
-  void (*on_game_start)();
- 
-};
-#endif // 0
-
-
 /** 
  * ソケットのコールバック関数.
  * 行を切り出して、recv_list に追加する. 
@@ -501,7 +291,7 @@ static gboolean on_socket_read( GIOChannel* source,
                                 GIOCondition condition,
                                 gpointer data )
 {
-  if ( !recv_buffer.read_from_server(server_fd) ) {
+  if ( !recv_buffer->read() ) {
     // network_log_append_message(_("server shutdown or an error occured.\n"));
     disconnect_server();
     return FALSE;
@@ -509,7 +299,7 @@ static gboolean on_socket_read( GIOChannel* source,
 
   GtkWidget* dialog;
 
-  while ( recv_buffer.recv_list.size() > 0) {
+  while ( recv_buffer->recv_list.size() > 0) {
     switch (wait_state) {
     case 1:
       // ログイン待ち
@@ -525,17 +315,20 @@ static gboolean on_socket_read( GIOChannel* source,
 
       gtk_widget_hide(connect_status_dialog);
 
-      // 受けるか拒否するかだけ
-      dialog = create_network_game_dialog();
-      if ( gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK ) {
-        csa_send_logout();
-        disconnect_server();
+      if ( !connect_arg->accept_immediately ) {
+	// 受けるか拒否するかだけ
+	dialog = create_network_game_dialog();
+	if ( gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK ) {
+	  csa_send_logout();
+	  disconnect_server();
+	  gtk_widget_hide(dialog);
+	  return FALSE;
+	}
 	gtk_widget_hide(dialog);
-        return FALSE;
       }
-      gtk_widget_hide(dialog);
 
-      csa_send("AGREE\n");
+      daemon_canvas_change_mode(g_canvas, D_CANVAS_MODE_PRE_GAME);
+      conn_send_str(server_fd, "AGREE\n");
       wait_state = 3;
       break;
     case 21:
@@ -546,6 +339,7 @@ static gboolean on_socket_read( GIOChannel* source,
     case 3:
       // ゲーム開始待ち
       if (state_3_read()) {
+	// ゲーム開始
 	g_signal_emit_by_name(G_OBJECT(g_canvas->window), 
 			      "notify::startgame", NULL);
 	return FALSE; // watchをいったん終了
@@ -556,6 +350,7 @@ static gboolean on_socket_read( GIOChannel* source,
         disconnect_server(); 
         daemon_messagebox(g_canvas, _("reject from the opponent"), 
 			  GTK_MESSAGE_INFO);
+	daemon_canvas_change_mode(g_canvas, D_CANVAS_MODE_EDIT);
         return FALSE;
       }
       break;
@@ -657,7 +452,7 @@ int connect_to_server(const string& hostname, const string& service)
 static int csa_send_login(const string& username, const string& password)
 {
   string buf = "LOGIN " + username + " " + password + "\n";
-  csa_send(buf.c_str());
+  conn_send_str(server_fd, buf.c_str());
   return 1;
 }
 
@@ -667,7 +462,7 @@ static int csa_send_login(const string& username, const string& password)
  */
 void csa_send_logout()
 {
-  csa_send("LOGOUT\n");
+  conn_send_str(server_fd, "LOGOUT\n");
 
 // TODO:  "LOGOUT:completed" を待つ
 }
@@ -675,7 +470,8 @@ void csa_send_logout()
 
 int csa_send_chudan_request()
 {
-  csa_send("%CHUDAN\n");
+  printf("%s: in\n", __func__); // DEBUG
+  conn_send_str(server_fd, "%CHUDAN\n");
   return 1;
 }
 
@@ -702,6 +498,9 @@ void* csa_connect_thread_main(void* arg_)
 /** ソケットを待つコールバック関数を登録する */
 void add_watch_socket()
 {
+  if (!recv_buffer)
+    recv_buffer = new RecvBuffer(server_fd);
+
   GIOChannel* channel = g_io_channel_unix_new(server_fd);
   io_watch = g_io_add_watch( channel, 
                              GIOCondition( G_IO_IN | G_IO_HUP | G_IO_ERR ),
@@ -714,53 +513,16 @@ void add_watch_socket()
 }
 
 
-void csa_login_and_wait_socket(const char* username_, const char* password)
+void csa_login_and_wait_socket(const ConnectThreadArg* arg)
 {
+  connect_arg = arg;
+
   add_watch_socket();
   wait_state = 1;  // サーバからのログイン許可を待つ
 
   // 引き続き、ログインする。
   // network_log_append_message("LOGIN %s\n", arg->username.c_str());
-  username = username_;
-  csa_send_login(username_, password);
-}
-
-
-/** 
- * 指し手を送信.
- * daemon_record_output_csa_te() を参考に。 
- */
-int csa_send_move(const BOARD* board, int is_sente, const DTe& te)
-{
-  if (te.special == DTe::RESIGN) {
-    csa_send("%TORYO\n");
-    return 1;
-  }
-
-  // 通常の指し手
-  char buf[100];
-  if ( !te.fm ) {
-    // 打ち
-    sprintf(buf, "%c%d%d%d%d%s\n",
-	    is_sente ? '+' : '-',
-	    0, 0, 
-	    te.to & 0xf, te.to >> 4,
-	    csa_koma_str[te.uti]);
-  }
-  else {
-    // 移動
-    PieceKind p = daemon_dboard_get_board(board, te.fm & 0xf, te.fm >> 4);
-    if ( te.nari ) p += 8;
-
-    sprintf(buf, "%c%d%d%d%d%s\n",
-	    is_sente ? '+' : '-',
-	    te.fm & 0xf, te.fm >> 4,
-	    te.to & 0xf, te.to >> 4,
-	    csa_koma_str[p & 0xf]);
-  }
-  csa_send(buf);
-
-  return 1;
+  csa_send_login(connect_arg->username, connect_arg->password);
 }
 
 
@@ -768,21 +530,31 @@ int csa_send_move(const BOARD* board, int is_sente, const DTe& te)
  * 手を送信した結果 (経過秒数など) を得る.
  * ブロックする。
  */
-INPUTSTATUS csa_wait_for_result( DTe* te, int* elapsed_time )
+INPUTSTATUS csa_wait_for_result( const BOARD* bo, DTe* te, int* elapsed_time )
 {
   printf("%s\n", __func__); // DEBUG
 
-  while (net_te_list.size() == 0)
+  while (net_queue.size() == 0) {
+    if ( gmGetGameStatus(&g_game) != 0 )
+      return gmGetGameStatus(&g_game);
+
     processing_pending( true );
+  }
 
-  NetTe net_te = net_te_list.front();
-  net_te_list.pop_front();
+  INPUTSTATUS r = conn_queue_front_move(net_queue, bo, te, elapsed_time);
+  if ( r == SI_CHUDAN_REMOTE || r == SI_TIMEOUT )
+    gmSetGameStatus(&g_game, r );
+  return r;
+}
 
-  // 千日手のことがある
-  *te = net_te.te;
-  *elapsed_time = net_te.sec;
-  printf("my time = %d\n", *elapsed_time); // DEBUG
 
-  // きわどいタイミングで時間切れのことがある
-  return gmGetGameStatus(&g_game);
+bool net_queue_is_pending()
+{
+  return net_queue.size() > 0;
+}
+
+
+int net_get_server_fd()
+{
+  return server_fd;
 }
