@@ -19,9 +19,9 @@
 */
 
 
-#define GTK_DISABLE_DEPRECATED 1
-#define GDK_DISABLE_DEPRECATED 1
-#define G_DISABLE_DEPRECATED 1
+// #define GTK_DISABLE_DEPRECATED 1
+// #define GDK_DISABLE_DEPRECATED 1
+// #define G_DISABLE_DEPRECATED 1
 #include <config.h>
 
 #include <gtk/gtk.h>
@@ -34,7 +34,6 @@ G_END_DECLS
 #include "canvas.h"
 #include "misc.h"
 #include "si/si.h"
-#include "si/ui.h"
 #include <string>
 #include <stdlib.h>
 using namespace std;
@@ -81,7 +80,7 @@ int main( int argc, char* argv[] )
   textdomain(GETTEXT_PACKAGE);
 #endif
 
-  gtk_set_locale();
+  // gtk_set_locale();
   gtk_init(&argc, &argv);
 
   // add_pixmap_directory( PACKAGE_DATA_DIR "/" PACKAGE );
@@ -90,7 +89,12 @@ int main( int argc, char* argv[] )
     g_thread_init(NULL);
 
   // 思考ファイル
+#if EVAL_VERSION == 1
   string think_config_file = PACKAGE_DATA_DIR "/" PACKAGE "/01.json";
+#else
+  string think_config_file = PACKAGE_DATA_DIR "/" PACKAGE "/02ai.txt";
+#endif 
+
   int ch;
   while( (ch = getopt(argc, argv, "c:")) != -1 ) {
     switch (ch) {
@@ -100,7 +104,6 @@ int main( int argc, char* argv[] )
     default:
       usage();
       exit(1);
-      break;
     }
   }
 
@@ -134,9 +137,12 @@ int main( int argc, char* argv[] )
   // 主ウィンドウを開く
   GtkWidget* window = GTK_WIDGET( 
                         gtk_builder_get_object(interface_builder, "window") );
-  g_canvas = daemon_canvas_new(window);
+  g_canvas = new Canvas( window );
+  if ( !g_canvas ) {
+    si_abort( "No enough memory in daemon_canvas_new()." );
+  }
 
-  set_pending_function(pending_loop);
+  si_set_pending_function(pending_loop);
 
   gtk_widget_show_all(window);
   gtk_main();
