@@ -19,9 +19,9 @@
 */
 
 
-#define GTK_DISABLE_DEPRECATED 1
-#define GDK_DISABLE_DEPRECATED 1
-#define G_DISABLE_DEPRECATED 1
+// #define GTK_DISABLE_DEPRECATED 1
+// #define GDK_DISABLE_DEPRECATED 1
+// #define G_DISABLE_DEPRECATED 1
 
 #include <gtk/gtk.h>
 #include <sys/time.h>
@@ -33,73 +33,26 @@
 using namespace std;
 
 
-/**
- * XPM のデータを読み込んで GdkPixmap にして返す。
- */
-GdkPixmap* load_pixmap_from_data(GtkWidget* window, char **data) 
-{
-  GdkPixmap *pixmap;
 
-  /* 画像を読む */
-  pixmap = gdk_pixmap_create_from_xpm_d(window->window, NULL, NULL, data);
-  if (!pixmap) {
-    si_abort("No enough memory in load_pixmap_from_data(). im is NULL\n");
-  }
+/** 画像を拡大する */
+cairo_surface_t* load_pixmap_from_imlib_image_scaled( cairo_format_t format,
+                      const GdkPixbuf* im, int width, int height ) 
+{
+  // GdkPixbuf *im2;
+  cairo_surface_t* pixmap = cairo_image_surface_create( format, width, height );
+  assert( pixmap );
+
+  cairo_t* cr = cairo_create( pixmap ); // dest
+  cairo_scale( cr, ((double) width) / gdk_pixbuf_get_width(im), 
+   	           ((double) height) / gdk_pixbuf_get_height(im) );
+  gdk_cairo_set_source_pixbuf( cr, im, 0, 0 );
+  cairo_rectangle( cr, 0, 0, width, height );
+  cairo_fill( cr );
+  cairo_destroy( cr );
 
   return pixmap;
 }
 
-GdkPixmap* load_pixmap_from_data_scaled(GtkWidget *window, const char** data,
-					gint w,
-					gint h) {
-  GdkPixbuf *im;
-  GdkPixbuf *im2;
-  GdkPixmap *pixmap;
-
-  /* 画像を読む */
-  im = gdk_pixbuf_new_from_xpm_data(data);
-  if (im == NULL) {
-    si_abort("No enough memory in load_pixmap_from_data_scaled(). im is NULL\n");
-  }
-
-  pixmap = load_pixmap_from_imlib_image_scaled(window, im, w, h);
-  g_object_unref(im);
-
-  return pixmap;
-}
-
-GdkPixmap* load_pixmap_from_imlib_image_scaled(GtkWidget *window, 
-					       GdkPixbuf *im,
-					       gint w,
-					       gint h) 
-{
-  GdkPixbuf *im2;
-  GdkPixmap* pixmap = NULL;
-
-  /* 画像を読む */
-  im2 = gdk_pixbuf_scale_simple(im, w, h, GDK_INTERP_NEAREST);
-  if (im2 == NULL) {
-    si_abort("No enough memory in load_pixmap_from_data_scaled(). im2 is NULL\n");
-  }
-  gdk_pixbuf_render_pixmap_and_mask(im2, &pixmap, NULL, 0);
-  g_object_unref(im2);
-
-  return pixmap;
-}
-
-
-void set_color(GdkGC* gc, guint16 r, guint16 g, guint16 b)
-{
-  GdkColor color;
-
-  color.red = r, color.green = g, color.blue = b;
-  if (!gdk_colormap_alloc_color(gdk_colormap_get_system(), &color, FALSE, TRUE)) {
-    printf("warning: fail gdk_colormap_alloc_color().\n");
-  }
-  gdk_gc_set_background(gc, &color);
-  gdk_gc_set_foreground(gc, &color);
-}
-
 
 /**
  * tv と now の時間差が DOUBLECLICKTIMEOUT 以内の場合は TRUE を返す。
@@ -140,7 +93,8 @@ gboolean check_timeval(const struct timeval* tv, const struct timeval* now)
  * @param b
  * @return 重なっていれば TRUE 、そうでなければ FALSE を返す。
  */
-gboolean is_on_rectangle(GdkRectangle* a, GdkRectangle* b) {
+gboolean is_on_rectangle( const GdkRectangle* a, const GdkRectangle* b ) 
+{
   if (a->x <= b->x && b->x <= a->x + a->width &&
       a->y <= b->y && b->y <= a->y + a->height) {
     return TRUE;
@@ -233,38 +187,3 @@ int daemon_canvas_yes_no_dialog(Canvas* canvas, const char* message)
 
   return reply;
 }
-
-
-/** 
- * string を返す sprintf().
- * エラーのとき例外を投げる
- */
-string str_format( const char* format, ... )
-{
-  int bufsiz = 100;
-  char* buf = NULL;
-
-  while (true) {
-    buf = static_cast<char*>(realloc(buf, bufsiz));
-    if (!buf)
-      throw runtime_error("no enough memory");
-
-    va_list ap;
-    va_start(ap, format);
-    int r = vsnprintf(buf, bufsiz, format, ap);
-    va_end(ap);
-
-    if ( r < 0 ) {
-      free(buf);
-      throw runtime_error(format);
-    }
-    else if ( r < bufsiz ) { // vsnprintf()の戻り値は'\0'を含まない長さ
-      string result = buf;
-      free(buf);
-      return result;
-    }
-
-    // 伸ばす
-    bufsiz = r + 1;
-  }
-}
