@@ -28,26 +28,23 @@
 /** ダブルクリックと判定するまでの時間(ミリ秒) */
 #define DOUBLECLICKTIMEOUT 400
 
-GdkPixmap* load_pixmap_from_data(GtkWidget* window, char **data);
-GdkPixmap* load_pixmap_from_data_scaled(GtkWidget* window, const char** data,
-					gint w,
-					gint h);
-GdkPixmap* load_pixmap_from_imlib_image_scaled(GtkWidget* window,
-					       GdkPixbuf *im,
-					       gint w,
-					       gint h);
 
-void set_color(GdkGC* gc, guint16 r, guint16 g, guint16 b);
+cairo_surface_t* load_pixmap_from_imlib_image_scaled( cairo_format_t format,
+             const GdkPixbuf* im, int width, int height );
+
+// void set_color_rgb(cairo_t* cr, guint16 r, guint16 g, guint16 b);
 
 gboolean check_timeval(const struct timeval* tv, const struct timeval* now);
 
-gboolean is_on_rectangle(GdkRectangle* a, GdkRectangle* b);
+gboolean is_on_rectangle( const GdkRectangle* a, const GdkRectangle* b);
 
 // void pending_loop( bool may_block );
 
 gboolean daemon_on_rectangle(GdkRectangle *rect, GdkPoint *point);
 
-std::string str_format( const char* format, ... );
+struct Canvas;
+void daemon_messagebox( Canvas* canvas, const char* s,
+				                    GtkMessageType mes_type );
 
 #endif /* _MISC_H_ */
 
