@@ -26,7 +26,6 @@
 #include <time.h>
 #include "si.h"
 #include "si-think.h"
-#include "ui.h"
 #include <algorithm>
 
 #define __STDC_FORMAT_MACROS 1
@@ -112,9 +111,11 @@ void bo_init(BOARD* bo)
       bo->long_attack[1][sq].count = 0;
     }
   }
-  
+
+#if EVAL_VERSION == 1  
   bo->point[0] = 0;
   bo->point[1] = 0;
+#endif // EVAL_VERSION == 1
 
   // 駒はすべて駒箱に
   for (i = 0; i < 9; i++)
@@ -127,12 +128,6 @@ void bo_init(BOARD* bo)
   }
   bo->reappearing_times.clear();
 
-  bo->total_time[SENTE] = 0;
-  bo->total_time[GOTE] = 0;
-  bo->limit_time[SENTE] = DEFAULT_LIMIT_TIME;
-  bo->limit_time[GOTE] = DEFAULT_LIMIT_TIME;
-  bo->before_think_time = 0;
-
   bo->next = 0;
 }
 
@@ -146,14 +141,6 @@ void boPlayInit(BOARD* bo)
 {
   boSetToBOARD(bo);
   
-  /* 消費時間に 0 をセットする */
-  bo->total_time[0] = 0;
-  bo->total_time[1] = 0;
-  
-  /* 持ち時間に1500秒(25分)をセットする */
-  bo->limit_time[0] = 1500;
-  bo->limit_time[1] = 1500;
-
   bo->mseq.count = 0;
   bo->reappearing_times.clear();
 }
@@ -175,13 +162,14 @@ static void pbPop( BOARD* bo, PieceKind p )
 
   /* oh */
   if (p == 0x08 || p == 0x18 ) {
-    assert( bo->pbox[8] >= 1 );
     bo->pbox[ 8 ]--;
+
+    assert( bo->pbox[8] >= 0 );
     return;
   }
 
-  assert( bo->pbox[p & 7] >= 1 );
   bo->pbox[p & 7]--;
+  assert( bo->pbox[p & 7] >= 0 );
 }
 
 
@@ -360,16 +348,18 @@ void boSetToBOARD(BOARD* bo)
     }
   }
 
-  if (bo->next) // 手番 = 後手
-    bo->hash_all = bo->key ^ ~ inhand_hash;
-  else
+  //  if (bo->next) // 手番 = 後手
+  //    bo->hash_all = bo->key ^ ~ inhand_hash;
+  //  else
     bo->hash_all = bo->key ^ inhand_hash;
 
+#if EVAL_VERSION == 1
   // 得点を設定
   if ( bo->king_xy[0] && bo->king_xy[1] ) {
     bo->point[0] = grade(bo, 0);
     bo->point[1] = grade(bo, 1);
   }
+#endif // EVAL_VERSION == 1
 }
 
 
@@ -395,12 +385,11 @@ static void inc_inhand(BOARD* bo, PieceKind p )
   assert( p != 8 && p != 0x18 );
 
   int count = ++bo->inhand[bo->next][p & 0x07];
-  // pbPush( &bo->pb, p );
 
 #ifdef USE_HASH  
-  if (bo->next) 
-    bo->hash_all ^= ~ hashseed_inhand[ bo->next][p & 7][ count ];
-  else
+  //  if (bo->next) 
+  //    bo->hash_all ^= ~ hashseed_inhand[ bo->next][p & 7][ count ];
+  //  else
     bo->hash_all ^= hashseed_inhand[ bo->next][p & 7][ count ];    
 #endif
 }
@@ -412,14 +401,13 @@ static void inc_inhand(BOARD* bo, PieceKind p )
 static void dec_inhand(BOARD* bo, PieceKind p)
 {
 #ifdef USE_HASH
-  if (bo->next)
-    bo->hash_all ^= ~ hashseed_inhand[ bo->next][p & 7][ bo->inhand[bo->next][p & 7]];
-  else
+  //  if (bo->next)
+  //    bo->hash_all ^= ~ hashseed_inhand[ bo->next][p & 7][ bo->inhand[bo->next][p & 7]];
+  //  else
     bo->hash_all ^= hashseed_inhand[ bo->next][p & 7][ bo->inhand[bo->next][p & 7]];
 #endif
 
   bo->inhand[bo->next][p & 7]--;
-  // pbPop( &bo->pb, p );
 }
 
 
@@ -658,8 +646,18 @@ static void remove_piece_with_kiki(BOARD* bo, Xy xy)
 #ifdef NDEBUG
   #define assert_move(bo, te)  (static_cast<void>(0))
 #else
-static void assert_move(const BOARD* bo, const TE& te)
+static void assert_move(BOARD* bo, const TE& te)
 {
+#if 0
+  // とても重い
+  uint64_t orig_key = bo->key;
+  uint64_t orig_hash_all = bo->hash_all;
+  boSetToBOARD(bo);
+  assert(orig_key == bo->key);
+  assert(orig_hash_all == bo->hash_all);
+#endif // 0
+
+  // 指し手の妥当性
   assert( 1 <= (te_to(te) & 0x0f) && (te_to(te) & 0x0f) <=9 );
   assert( 1 <= (te_to(te) >> 4) && (te_to(te) >> 4) <=9 );
   if ( !te_is_put(te) ) {
@@ -771,6 +769,15 @@ static void bo_update_pin(BOARD* bo, int king_side, Xy from, Xy to)
 #else
 static void assert_post_move(BOARD* bo, const TE& te)
 {
+#if 0
+  // とても重い
+  uint64_t orig_key = bo->key;
+  uint64_t orig_hash_all = bo->hash_all;
+  boSetToBOARD(bo);
+  assert(orig_key == bo->key);
+  assert(orig_hash_all == bo->hash_all);
+#endif // 0
+
   if ( bo_is_checked(bo, 1 - bo->next) ) {
     printf("internal error: still checked.\n");
 
@@ -828,7 +835,9 @@ int boMove( BOARD* bo, const TE& te )
 
     // 移動元.
     // printf("from:%d ", -add_grade(bo, te_from(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] -= add_grade(bo, te_from(te), next );
+#endif // EVAL_VERSION == 1
     remove_piece_with_kiki(bo, te_from(te));
 
     c = bo->board[ te_to(te) ];
@@ -838,11 +847,15 @@ int boMove( BOARD* bo, const TE& te )
       // 持ち駒に加える
       inc_inhand(bo, c );
       // printf("inhand:%d ", +score_inhand(bo, c & 7, next));
+#if EVAL_VERSION == 1
       bo->point[next] += score_inhand( bo, c & 7, next );
+#endif // EVAL_VERSION == 1
 
       // 盤上から駒を取り除く
       // printf("remove_to(op):%d ", -add_grade(bo, te_to(te), (next == 0)));
+#if EVAL_VERSION == 1
       bo->point[1 - next] -= add_grade(bo, te_to(te), (next == 0));
+#endif // EVAL_VERSION == 1
       remove_piece(bo, te_to(te));
 
       /* 取られる駒を覚えておく */
@@ -862,26 +875,33 @@ int boMove( BOARD* bo, const TE& te )
     else
       put_piece_with_kiki(bo, te_to(te), p );
     // printf("to:%d ", add_grade(bo, te_to(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] += add_grade(bo, te_to(te), next );
+#endif // EVAL_VERSION == 1
   } 
   else {
     // 駒打ちの場合
     
     // 持ち駒を減らす
     // printf("dec_inhand:%d ", -score_inhand(bo, te_pie(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] -= score_inhand( bo, te_pie(te), next );
+#endif // EVAL_VERSION == 1
     dec_inhand(bo, te_pie(te));
 
     // 駒を置く
     put_piece_with_kiki(bo, te_to(te), te_pie(te) + (next << 4) );
     // printf("put:%d ", add_grade(bo, te_to(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] += add_grade(bo, te_to(te), next);
+#endif // EVAL_VERSION == 1
     
     bo->tori[ bo->mseq.count - 1 ] = 0;
   }
 
   boToggleNext(bo);
 
+#if EVAL_VERSION == 1
   if ((p & 0xf) == OH) {
     bo->point[0] = grade(bo, SENTE);
     bo->point[1] = grade(bo, GOTE);
@@ -889,6 +909,7 @@ int boMove( BOARD* bo, const TE& te )
     bo_update_pin(bo, 1 - bo->next, te_from(te), te_to(te) );
 #endif
   }
+#endif // EVAL_VERSION == 1
 
   if ( bo_is_checked(bo, bo->next) )
     te_add_hint( &bo->mseq.te[bo->mseq.count - 1], TE_CHECK );
@@ -924,7 +945,9 @@ void boBack( BOARD* bo )
   PieceKind cap;
 
   boToggleNext(bo);
+#if EVAL_VERSION == 1
   int next = bo->next;
+#endif // EVAL_VERSION == 1
 
   const TE& te = bo->mseq.te[ --bo->mseq.count ];
   cap = bo->tori[ bo->mseq.count ];
@@ -937,7 +960,9 @@ void boBack( BOARD* bo )
 
     /* 盤面評価点数から te_to(te) の位置の駒の点数を引く */
     // printf("to:%d ", -add_grade(bo, te_to(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] -= add_grade(bo, te_to(te), next );
+#endif // EVAL_VERSION == 1
 
     if (cap) {
       // 駒を取っていた場合
@@ -945,13 +970,18 @@ void boBack( BOARD* bo )
 
       // 持ち駒から減らす
       // printf("inhand:%d ", -score_inhand(bo, cap & 7, next));
+#if EVAL_VERSION == 1
       bo->point[next] -= score_inhand(bo, cap & 7, next);
+#endif // EVAL_VERSION == 1
+
       dec_inhand(bo, cap);
 
       // 駒を戻す
       put_piece(bo, te_to(te), cap);
       // printf("restore(op):%d ", add_grade(bo, te_to(te), (next == 0)));
+#if EVAL_VERSION == 1
       bo->point[1 - next] += add_grade(bo, te_to(te), (next == 0));
+#endif // EVAL_VERSION == 1
     }
     else
       remove_piece_with_kiki(bo, te_to(te));
@@ -961,22 +991,29 @@ void boBack( BOARD* bo )
       p &= 0x17;
     put_piece_with_kiki(bo, te_from(te), p );
     // printf("from:%d ", add_grade(bo, te_from(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] += add_grade(bo, te_from(te), next);
+#endif // EVAL_VERSION == 1
   } 
   else {
     /* 駒打ちだった場合 */
 
     // 取り除く
     // printf("remove:%d ", -add_grade(bo, te_to(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] -= add_grade(bo, te_to(te), next);
+#endif // EVAL_VERSION == 1
     remove_piece_with_kiki(bo, te_to(te));
 
     // 持ち駒に戻す
     inc_inhand(bo, te_pie(te) );
     // printf("to_inhand:%d ", +score_inhand(bo, te_pie(te), next));
+#if EVAL_VERSION == 1
     bo->point[next] += score_inhand(bo, te_pie(te), next);
+#endif // EVAL_VERSION == 1
   }
-  
+
+#if EVAL_VERSION == 1  
   if ( (p & 0xf) == OH ) {
     bo->point[0] = grade(bo, SENTE);
     bo->point[1] = grade(bo, GOTE);
@@ -984,6 +1021,7 @@ void boBack( BOARD* bo )
     bo_update_pin(bo, bo->next, te_to(te), te_from(te) );
 #endif
   }
+#endif // EVAL_VERSION == 1
 
   // printf("\n%s: out: point = (%d, %d)\n", __func__, bo->point[0], bo->point[1]);
 }
@@ -1185,127 +1223,6 @@ void bo_pass( BOARD* bo )
 }
 
 
-/**
- * te の手が成りが可能か調べる。
- * @param bo 対象のBOARD
- * @param te 対象のTE
- * @return 成れないときは 0, どちらも可能 1, 成るしかない 2
- */
-int can_move_promote(const BOARD* bo, const TE& te) 
-{
-  PieceKind p;
-  int next;
-  int fml ,tol;
-
-  assert(bo != NULL);
-
-  if ( te_is_put(te) )
-    return 0;
-
-  assert( te_from(te) >= 0x11 && te_from(te) <= 0x99 );
-  assert( bo->board[te_from(te)] != 0 );
-
-  p = boGetPiece(bo, te_from(te) );
-  next = getTeban(p);
-
-  p &= 0x0F;
-
-  /* 王と金は成れない。戻る */
-  if (p == OH || p == KIN)
-    return 0;
-
-  /* もう成っている駒はなれない。戻る */
-  if ((p & 0x08) == 0x08)
-    return 0;
-
-  fml = (te_from(te) >> 4);
-  tol = (te_to(te) >> 4);
-
-  if ( next == SENTE ) {
-    if ( (p == FU || p == KYO) && tol <= 1 )
-      return 2;
-    else if ( p == KEI && tol <= 2 )
-      return 2;
-    else if (fml <= 3 || tol <= 3)
-      return 1;
-  }
-  else {
-    if ( (p == FU || p == KYO) && 9 <= tol )
-      return 2;
-    else if ( p == KEI && 8 <= tol )
-      return 2;
-    else if (7 <= fml || 7 <= tol)
-      return 1;
-  }
-
-  return 0;
-}
-
-
-/**
- * 千日手 (同一局面4回) になる手か？
- * @param bo 現在の局面
- * @param te これから指そうとする手
- * @param forbidden 連続王手の千日手のときtrueに出力する
- *
- * @return 千日手のときtrue
- */
-bool te_is_sennichite(const BOARD* bo, const TE& te, bool* forbidden)
-{
-  BOARD* tmp_bo = bo_dup(bo);
-  boMove_mate(tmp_bo, te);
-
-  if ( bo_repetition_count(tmp_bo) >= 3 ) {
-    // 連続王手の千日手かどうか
-
-    // http://www.shogi.or.jp/faq/
-    // 全部の手が王手のとき以外は、通常の千日手扱い
-
-    printBOARD(bo);
-    printf("move = %s = sennichite\n", te_str(te, *bo).c_str() ); // DEBUG
-
-    freeBOARD(tmp_bo);
-    return true;
-  }
-
-  freeBOARD(tmp_bo);
-  return false;
-}
-
-
-#if 0
-/**
- * 現在の時刻を記録する。
- * (思考時間の測定用)
- * @param bo 対象のBOARD
- */
-void boSetNowTime(BOARD *bo) {
-  bo->before_think_time = time(NULL);
-}
-
-
-/**
- * 消費した時間を記録する。
- * @param bo 対象のBOARD
- */ 
-void boSetUsedTime(BOARD *bo) 
-{
-  time_t used_time;
-  
-  used_time = time(NULL) - bo->before_think_time;
-#ifdef DEBUG
-  assert( 0 <= used_time );
-#endif /* DEBUG */
-  if ( used_time <= 0 ) {
-    used_time = 1;
-  }
-  
-  bo->total_time[bo->next] += used_time;
-  // bo->used_time[bo->mi.count] = used_time;
-}
-#endif // 0
-
-
 static int bo_is_correct_mate(const BOARD* bo, Xy* oh_xy)
 {
   int piece[9]; // 駒種ごとの駒の数
@@ -1474,15 +1391,16 @@ int boCheckJustMate(const BOARD* bo)
 }
 
 
-extern const char* CSA_PIECE_NAME[];
-
-/** 盤面をコンソールに出力 */
-void printBOARD(const BOARD* brd)
+/** 
+ * 盤面をコンソールに出力.
+ * \todo front = 1 のときの表示
+ */
+void printBOARD(const BOARD* brd, int front)
 {
-  int x, y;
-
   assert( brd != NULL );
 
+  int x, y;
+
   for (y = 1; y <= 9; y++) {
     printf("P%d", y);
     for (x = 9; x >= 1; x--) {
@@ -1517,6 +1435,59 @@ void printBOARD(const BOARD* brd)
     }
   }
   printf( "%c\n", brd->next == 0 ? '+' : '-' );
+#ifndef NDEBUG
   printf( "'key = %" PRIx64 ", all = %" PRIx64 "\n", brd->key, brd->hash_all );
+#endif // !NDEBUG
 }
 
+
+static const char* fix_map = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+/** 盤面を固定長の文字列にする */
+string bo_to_fixed_str( const BOARD* bo )
+{
+  char s[104];
+
+  // 盤上
+  for (int y = 1; y <= 9; y++) {
+    for (int x = 1; x <= 9; x++) 
+      s[ (y - 1) * 10 + x - 1 ] = fix_map[ bo->board[(y << 4) + x] ];
+    s[ (y - 1) * 10 + 9 ] = '/';
+  }
+
+  // 持ち駒
+  for (int j = 0; j < 2; j++) {
+    for (int i = 1; i <= 7; i++)
+      s[ 89 + j * 7 + (i - 1) ] = 'a' + bo->inhand[j][i];
+  }
+
+  // 手番
+  s[ 103 ] = !bo->next ? '+' : '-';
+
+  return string(s, 104);
+}
+
+
+/** 固定長の文字列から復元する */
+bool bo_set_by_fixed_str( BOARD* bo, const string& str )
+{
+  for (int y = 1; y <= 9; y++) {
+    for (int x = 1; x <= 9; x++) {
+      const char* p = strchr( fix_map, str[(y - 1) * 10 + x - 1] );
+      if (!p)
+	return false;
+      bo->board[(y << 4) + x] = p - fix_map;
+    }
+  }
+
+  // 持ち駒
+  for (int j = 0; j < 2; j++) {
+    for (int i = 1; i <= 7; i++)
+      bo->inhand[j][i] = str[89 + j * 7 + (i - 1)] - 'a';
+  }
+
+  bo->next = str[103] == '+' ? 0 : 1;
+
+  boSetToBOARD(bo);
+  return true;
+}
