@@ -28,8 +28,6 @@
 #include <stddef.h> // offsetof
 #include "si.h"
 #include "si-think.h"
-#include "ui.h"
-#include "taboo_list.h"
 #include <iostream>
 using namespace std;
 
@@ -129,14 +127,10 @@ void freeHASH()
  */
 uint64_t hsGetHashKey(const BOARD* bo)
 {
-  uint64_t key;
-  int i;
-
-#ifdef DEBUG
   assert( bo != NULL );
-#endif /* DEBUG */
 
-  key = 0;
+  uint64_t key = 0;
+  int i;
 
   for ( i = 1; i <= 81; i++ ) {
     // if ( bo->board[arr_xy[i]] )
@@ -389,7 +383,7 @@ void assert_tt_put(const BOARD* bo, const TE& best_move)
  * @param best_move 最善手. 投了はNULL
  */
 void hsPutBOARD( int tag, const BOARD* bo, 
-                 int32_t phi, int32_t delta, int te_count,
+                 int32_t phi, int32_t delta, int rest_depth,
                  const TE& best_move )
 {
   assert_tt_put(bo, best_move);
@@ -402,7 +396,7 @@ void hsPutBOARD( int tag, const BOARD* bo,
   if (!old_ent) {
     HASH n;
     n.key = bo->key;
-    n.ent[0] = make_v(bo, phi, delta, te_count, best_move);
+    n.ent[0] = make_v(bo, phi, delta, rest_depth, best_move);
     tcmapput( ttable_mate[tag], &bo->key, sizeof(bo->key), 
 	      &n, sizeof(HASH) );
     return;
@@ -438,7 +432,7 @@ void hsPutBOARD( int tag, const BOARD* bo,
     }
   }
   else if ( phi >= VAL_INF && delta == 0 ) {
-    // 不詰み. 持ち駒がより少ない不詰み情報は捨ててよい
+    // 負け. 持ち駒がより少ない不詰み情報は捨ててよい
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
 
@@ -461,7 +455,7 @@ void hsPutBOARD( int tag, const BOARD* bo,
       const EntVariant* ent = old_ent->ent + i;
 
       if ( ent->phi == 0 && ent->delta >= VAL_INF ) {
-        // 詰みがある.
+        // すでに勝ちが登録されている
         if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
           to_add = false;
           new_ent->ent[count++] = *ent;
@@ -478,9 +472,9 @@ void hsPutBOARD( int tag, const BOARD* bo,
           new_ent->ent[count++] = *ent;
       }
       else {
-        if ( ent->te_count < te_count || !hsCmpPiece(ent, bo) ) {
+        if ( /*ent->te_count < te_count ||*/ !hsCmpPiece(ent, bo) ) {
           // 深さで判定するのは不味い. 深いところから戻りながら更新していく
-          // skip
+          // 既存のは捨てる. skip
         }
         else
           new_ent->ent[count++] = *ent;
@@ -490,7 +484,7 @@ void hsPutBOARD( int tag, const BOARD* bo,
 
   // 追加すべきときだけ実際に書き込む
   if (to_add) {
-    new_ent->ent[count++] = make_v(bo, phi, delta, te_count, best_move);
+    new_ent->ent[count++] = make_v(bo, phi, delta, rest_depth, best_move);
     tcmapput( ttable_mate[tag], &bo->key, sizeof(bo->key),
               new_ent, HASH_HEADER_SIZE + HASH_ENT_SIZE * count );
 
@@ -530,12 +524,12 @@ void assert_post_tt_get(const BOARD* bo, const TE& best_move)
  * @param phi 攻め方ノードで証明数.
  * @param delta 攻め方ノードで反証数.
  * @param te 最善手. NULLでもよい
- * @param te_count 手数. 深さではない
+ * @param rest_depth 探索深さ。逃れの場合だけ考慮する。
  *
  * @return 盤面が見つかったら 1 、見つからなかったら 0 を返す。
  */
 int hsGetBOARD( int tag, const BOARD* bo, 
-                int32_t* phi, int32_t* delta, TE* te, int te_count )
+                int32_t* phi, int32_t* delta, TE* te, int rest_depth )
 {
   assert( phi && delta );
 
@@ -548,10 +542,14 @@ int hsGetBOARD( int tag, const BOARD* bo,
   for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
     const EntVariant* ent = cntr->ent + i;
 
-    if ( (ent->phi == 0 && ent->delta >= VAL_INF) || 
-	 (ent->te_count >= te_count && 
-	          ent->phi == 0 && ent->delta >= VAL_MAY_MATCH) ) {
-      // 現局面のほうが優越してれば詰み
+    if ( ent->phi == 0 || ent->delta >= VAL_MAY_MATCH ) {
+      // 勝ち？
+
+      // 逃れの場合だけ特別扱い
+      if (ent->delta < VAL_INF && ent->te_count < rest_depth )
+	continue;
+
+      // 手番プレイヤーについて現局面 (bo) のほうが優越してれば勝ち
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
 	*phi = ent->phi; *delta = ent->delta;
         if (te) {
@@ -560,11 +558,13 @@ int hsGetBOARD( int tag, const BOARD* bo,
 	}
 	return 1;
       }
+      continue;
     }
-    else if ( (ent->phi >= VAL_INF && ent->delta == 0) ||
-	      (ent->te_count >= te_count && 
-	               ent->phi >= VAL_MAY_MATCH && ent->delta == 0) ) {
-      // 不詰み
+    else if ( ent->phi >= VAL_MAY_MATCH || ent->delta == 0 ) {
+      if ( ent->phi < VAL_INF && ent->te_count < rest_depth )
+	continue;
+
+      // 負け
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
 	*phi = ent->phi; *delta = ent->delta;
         if (te) {
@@ -578,15 +578,13 @@ int hsGetBOARD( int tag, const BOARD* bo,
 	}
 	return 1;
       }
+      continue;
     }
     else {
-      if ( (ent->phi >= VAL_MAY_MATCH && ent->delta == 0) ||
-           (ent->phi == 0 && ent->delta >= VAL_MAY_MATCH) )
-        continue;
 #if 0
       if (ent->te_count < te_count)
         continue;
-#endif
+#endif // 0
 
       if ( !hsCmpPiece(ent, bo) ) {
 	*phi = ent->phi; *delta = ent->delta;
@@ -648,7 +646,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
   bool to_add = true;
   int i;
 
-  if ( phi >= +VAL_INF && delta <= -VAL_INF ) {
+  if ( phi >= +VAL_INF || delta <= -VAL_INF ) {
     // 勝ち
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
@@ -658,7 +656,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
         // skip
       }
       else if ( hsSuperiorBOARD(ent, bo) < 0 && 
-		(ent->phi >= +VAL_INF && ent->delta <= -VAL_INF) ) {
+		(ent->phi >= +VAL_INF || ent->delta <= -VAL_INF) ) {
         // 既存のも勝ち
         to_add = false;
         new_ent->ent[count++] = *ent;
@@ -667,7 +665,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
         new_ent->ent[count++] = *ent;
     }
   }
-  else if ( phi <= -VAL_INF && delta >= +VAL_INF ) {
+  else if ( phi <= -VAL_INF || delta >= +VAL_INF ) {
     // 負け
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
@@ -676,7 +674,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
         // skip
       }
       else if ( hsSuperiorBOARD(ent, bo) > 0 && 
-		(ent->phi <= -VAL_INF && ent->delta >= +VAL_INF) ) {
+		(ent->phi <= -VAL_INF || ent->delta >= +VAL_INF) ) {
         to_add = false;
         new_ent->ent[count++] = *ent;
       }
@@ -689,7 +687,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
     for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
       const EntVariant* ent = old_ent->ent + i;
 
-      if ( ent->phi >= +VAL_INF && ent->delta <= -VAL_INF ) {
+      if ( ent->phi >= +VAL_INF || ent->delta <= -VAL_INF ) {
         // 既存のエントリが勝ち
         if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
           to_add = false;
@@ -698,7 +696,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
         else
           new_ent->ent[count++] = *ent;
       }
-      else if ( ent->phi <= -VAL_INF && ent->delta >= +VAL_INF ) {
+      else if ( ent->phi <= -VAL_INF || ent->delta >= +VAL_INF ) {
         if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
           to_add = false;
           new_ent->ent[count++] = *ent;
@@ -749,7 +747,7 @@ void hsPutBOARDMinMax( const BOARD* bo, int32_t phi, int32_t delta,
  * 盤面の得点 (score) を取りだす。min/max検索用。
  * @param bo 対象のBOARD
  * @param score 得点を格納する変数のポインタ
- * @param te 見つかった指し手を格納する先
+ * @param te 見つかった指し手を格納する先. NULLのときは格納しない
  * @param te_count 手数. 深さではない
  *
  * @return 盤面が見つかったら 1 、見つからなかったら 0 を返す。
@@ -775,7 +773,7 @@ int hsGetBOARDMinMax( const BOARD* bo, PhiDelta* score, TE* te, int te_count )
   for (i = 0; i < (size - HASH_HEADER_SIZE) / HASH_ENT_SIZE; i++) {
     const EntVariant* ent = cntr->ent + i;
 
-    if ( ent->phi >= +VAL_INF && ent->delta <= -VAL_INF ) {
+    if ( ent->phi >= +VAL_INF || ent->delta <= -VAL_INF ) {
       // 勝ち.
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) < 0 ) {
         score->phi = ent->phi; score->delta = ent->delta;
@@ -789,7 +787,7 @@ int hsGetBOARDMinMax( const BOARD* bo, PhiDelta* score, TE* te, int te_count )
         return 1;
       }
     }
-    else if ( ent->phi <= -VAL_INF && ent->delta >= +VAL_INF ) {
+    else if ( ent->phi <= -VAL_INF || ent->delta >= +VAL_INF ) {
       // 負け
       if ( !hsCmpPiece(ent, bo) || hsSuperiorBOARD(ent, bo) > 0 ) {
         score->phi = ent->phi; score->delta = ent->delta;
