@@ -22,13 +22,19 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <signal.h> // sig_atomic_t
 #include "si.h"
 #include "si-think.h"
-#include "ui.h"
 #include "taboo_list.h"
 
-#define DEBUGLEVEL 0
-#define DEBUG_TESUKI_LEVEL 0
+#ifdef NDEBUG
+  // リリースモード
+  #define DEBUGLEVEL 0
+  #define DEBUG_TESUKI_LEVEL 0
+#else
+  #define DEBUGLEVEL 0
+  #define DEBUG_TESUKI_LEVEL 1
+#endif // NDEBUG
 
 #if DEBUGLEVEL >= 1
 #define __STDC_FORMAT_MACROS 1
@@ -42,7 +48,7 @@ using namespace std;
  * @return 見つかったときtrue 
  */
 static bool mate_lookup_hash( const TabooList<PhiDelta>* taboo_list, 
-                              int tag, const BOARD* bo, int const te_count,
+                              int tag, const BOARD* bo, int const rest_depth,
 			      TE* best_move,
                               int32_t* phi, int32_t* delta )
 {
@@ -74,7 +80,7 @@ static bool mate_lookup_hash( const TabooList<PhiDelta>* taboo_list,
     return true;
   }
 
-  if ( hsGetBOARD(tag, bo, phi, delta, best_move, te_count) )
+  if ( hsGetBOARD(tag, bo, phi, delta, best_move, rest_depth) )
     return true;
 
   *phi = 1;  // TODO: ありそうな局面はよい (低い) 値
@@ -135,9 +141,9 @@ static void set_offense_muda_uchi( const EndingSearchParam& param,
  */
 static bool offense_move( const TabooList<PhiDelta>* taboo_list, 
 			  const EndingSearchParam& param, 
-			  BOARD* node, 
+			  BOARD* node, int const rest_depth,
 			  TE& te,
-			  int32_t* child_phi, int32_t* child_delta )
+			  PhiDelta* child_score )
 {
   assert(node->next == param.offense_side);
 
@@ -156,9 +162,9 @@ static bool offense_move( const TabooList<PhiDelta>* taboo_list,
   boMove_mate( node, te );
   
   TE tt_move;
-  bool r = mate_lookup_hash(taboo_list, tt_tag(param), node, param.te_count,
+  bool r = mate_lookup_hash(taboo_list, tt_tag(param), node, rest_depth - 1,
 			    &tt_move,
-                            child_phi, child_delta);
+                            &child_score->phi, &child_score->delta);
   if (r) {
     if ( !teCmpTE(te, tt_move) )
       te_set_hint( &te, te_hint(tt_move) );
@@ -178,15 +184,17 @@ static bool offense_move( const TabooList<PhiDelta>* taboo_list,
 #endif // !NDEBUG
 
     MOVEINFO mi_uke;
-    *child_delta = uke(node, &mi_uke );
+    child_score->delta = uke(node, &mi_uke );
 
-    if (*child_delta == 0) {
+    if (child_score->delta == 0) {
+#if 0
       if ( te_is_put(te) && te_pie(te) == FU ) {
 	// 打ち歩詰め
 	boBack_mate( node );
 	return false;
       }
-      *child_phi = +VAL_INF;
+#endif // 0
+      child_score->phi = +VAL_INF;
       return true;
     }
 
@@ -220,15 +228,17 @@ static bool offense_move( const TabooList<PhiDelta>* taboo_list,
     // 王手はいつでもOK
     if ( bo_is_checked(node, 1 - param.offense_side) ) {
       MOVEINFO mi_uke;
-      *child_delta = uke(node, &mi_uke);
+      child_score->delta = uke(node, &mi_uke);
 
-      if (*child_delta == 0) {
+      if (child_score->delta == 0) {
+#if 0
 	if ( te_is_put(te) && te_pie(te) == FU ) {
 	  //打ち歩詰め
 	  boBack_mate( node );
 	  return false;
 	}
-	*child_phi = +VAL_INF;
+#endif // 0
+	child_score->phi = +VAL_INF;
       }
 
       return true;
@@ -267,7 +277,7 @@ static bool offense_move( const TabooList<PhiDelta>* taboo_list,
       boBack_mate(node); // パスだけ戻す
 
       // 詰みにくいほうが子ノードにとって好ましい
-      *child_delta = GAME_MATE_NODE_LIMIT - node_limit + 1;
+      child_score->delta = GAME_MATE_NODE_LIMIT - node_limit + 1;
       return true;
     }
     else {
@@ -284,6 +294,8 @@ static bool offense_move( const TabooList<PhiDelta>* taboo_list,
     if (attacked)
       return true;
     assert(0);
+
+    return true;
   }
 }
 
@@ -308,7 +320,7 @@ void TsumeroNogare::before_move(const BOARD* node, int offense_side_)
                                node->king_xy[1 - offense_side] );
 
     // 自玉の広さ
-    mkm = think_count_king_movables( node, 1 - offense_side );
+    mkm = think_count_king_movable2( node, 1 - offense_side );
 }
 
 
@@ -332,7 +344,7 @@ bool TsumeroNogare::promise( const BOARD* bo ) const
     r = true;
   
   // - 玉を広くする手
-  else if ( think_count_king_movables(bo, 1 - offense_side) > mkm )
+  else if ( think_count_king_movable2(bo, 1 - offense_side) > mkm )
     r = true;
 
   // - 玉が動く手
@@ -345,9 +357,9 @@ bool TsumeroNogare::promise( const BOARD* bo ) const
 
 static bool defense_move( const TabooList<PhiDelta>* taboo_list,
 			  const EndingSearchParam& param,
-			  BOARD* node,
+			  BOARD* node, int const rest_depth,
                           MOVEINFO* const mi, int index,
-			  int32_t* child_phi, int32_t* child_delta )
+			  PhiDelta* child_score )
 {
   assert( node->next == 1 - param.offense_side );
 
@@ -363,9 +375,9 @@ static bool defense_move( const TabooList<PhiDelta>* taboo_list,
   boMove_mate(node, mi->te[index]); // 受け側の手
 
   TE tt_move;
-  bool r = mate_lookup_hash(taboo_list, tt_tag(param), node, param.te_count,
+  bool r = mate_lookup_hash(taboo_list, tt_tag(param), node, rest_depth - 1,
 			    &tt_move,
-			    child_phi, child_delta);
+			    &child_score->phi, &child_score->delta);
   if (r) {
     if ( !teCmpTE(mi->te[index], tt_move) )
       te_set_hint( &mi->te[index], te_hint(tt_move) );
@@ -382,9 +394,9 @@ static bool defense_move( const TabooList<PhiDelta>* taboo_list,
     MOVEINFO mi_dmy;
 
     ohte(node, &mi_dmy);
-    *child_delta = mi_dmy.count;
-    if (*child_delta == 0)
-      *child_phi = +VAL_INF;
+    child_score->delta = mi_dmy.count;
+    if (child_score->delta == 0)
+      child_score->phi = +VAL_INF;
 
     return true;
   }
@@ -405,7 +417,8 @@ static bool defense_move( const TabooList<PhiDelta>* taboo_list,
 			&node_limit);
     if ( mr == CHECK_MATE ) {
       // 受けになっていない. 負け
-      *child_phi = 0; *child_delta = +VAL_INF;
+      child_score->phi = 0; 
+      child_score->delta = +VAL_INF;
       hsPutBOARD( tt_tag(param), node, 0, +VAL_INF, param.te_count, TE_NUL );
 
       if ( te_is_put(mi->te[index]) ) {
@@ -427,14 +440,14 @@ static bool defense_move( const TabooList<PhiDelta>* taboo_list,
  * 勝ち = (0, +VAL_INF), 負け = (+VAL_INF, 0)
  * 一通り展開するので、move ordering は効果なし
  */
-static void expand_and_best_move( const TabooList<PhiDelta>* taboo_list,
+static void mate_expand_and_best_move( const TabooList<PhiDelta>* taboo_list,
                                   const EndingSearchParam& param,
                                   BOARD* node,
                                   int rest_depth, int rest_node,
-                                  PhiDelta* best_child,
-                                  int32_t* child_phi_sum,
-                                  int32_t* second_child_delta,
-                                  TE* best_move )
+				       PhiDelta* best_child,
+				       int32_t* child_phi_sum,
+				       int32_t* second_child_delta,
+				       TE* best_move )
 {
   // 手を展開する
   MOVEINFO mi;
@@ -453,13 +466,14 @@ static void expand_and_best_move( const TabooList<PhiDelta>* taboo_list,
   *child_phi_sum = 0;
 
   if (!mi.count) {
+    // 負け
     te_clear(best_move);
     return;
   }
 
+  // 逃げ切った?
   if ( node->next != param.offense_side && 
-       (rest_depth <= 0 || rest_node <= 0) ) {
-    // 逃げ切った
+                                  (rest_depth <= 0 || rest_node <= 0) ) {
 #if DEBUGLEVEL >= 3
     if (param.start_tesuki >= DEBUG_TESUKI_LEVEL)
       printf("%d: unreach win.\n", rest_depth);
@@ -479,44 +493,44 @@ static void expand_and_best_move( const TabooList<PhiDelta>* taboo_list,
 #endif
 
   for (int i = 0; i < mi.count; i++) {
-    int32_t child_phi, child_delta;
+    PhiDelta child_score;
 
     if (node->next == param.offense_side) {
-      if ( !offense_move(taboo_list, param, node, mi.te[i], 
-			 &child_phi, &child_delta) ) {
+      if ( !offense_move(taboo_list, param, node, rest_depth, mi.te[i], 
+			 &child_score) ) {
 	continue;
       }
     }
     else {
-      if ( !defense_move(taboo_list, param, node, &mi, i,
-			 &child_phi, &child_delta) ) {
+      if ( !defense_move(taboo_list, param, node, rest_depth, &mi, i,
+			 &child_score) ) {
 	continue;
       }
     }
 
-    if (child_delta < best_child->delta) {
+    if (child_score.delta < best_child->delta) {
       // 最善手を更新
       *second_child_delta = best_child->delta;
-      best_child->phi = child_phi;
-      best_child->delta = child_delta;
+      *best_child = child_score;
       may_best = mi.te[i];
     }
-    else if (child_delta < *second_child_delta) {
+    else if (child_score.delta < *second_child_delta) {
       // 2位を更新
-      *second_child_delta = child_delta;
+      *second_child_delta = child_score.delta;
     }
 
-    *child_phi_sum = sum_step(*child_phi_sum, child_phi);
+    *child_phi_sum = sum_step(*child_phi_sum, child_score.phi);
 
     boBack_mate(node);
+
 #if DEBUGLEVEL >= 3
     if (param.start_tesuki >= DEBUG_TESUKI_LEVEL) {
       printf("%s=(%d,%d), ", te_str(mi.te[i], *node).c_str(), 
-	     child_phi, child_delta);
+	     child_score.phi, child_score.delta);
     }
 #endif
 
-    if (child_phi >= +VAL_MAY_MATCH)
+    if ( child_score.phi >= +VAL_MAY_MATCH)
       break; // nodeの手番側の勝ち確定
   }
 
@@ -559,6 +573,7 @@ static void set_muda_ai(int tag, const BOARD* node, int te_count)
 
 #ifdef USE_GUI
 extern GAME g_game;
+extern volatile sig_atomic_t gui_pending_counter;
 #endif // USE_GUI
 
 
@@ -592,8 +607,9 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
   te_clear(&prev_move);
 
 #ifdef USE_GUI
-  if ( !(*node_limit % 700) ) {
+  if ( gui_pending_counter ) {
     processing_pending( false );
+    gui_pending_counter = 0;
   }
 #endif // USE_GUI
 
@@ -602,16 +618,17 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
     if ( tesuki < param.start_tesuki ) {
       assert( tesuki == param.start_tesuki - 1 );
 
-      TE te_ret;
-      int r = bfs_hisshi(node, tesuki, 31, &te_ret, node_limit);
-      if (r == CHECK_MATE)
-        hsPutBOARD( tt_tag(param), node, 0, +VAL_INF, param.te_count, te_ret );
+      int r = bfs_hisshi(node, tesuki, 31, &best_move, node_limit);
+      if (r == CHECK_MATE) {
+        hsPutBOARD( tt_tag(param), node, 0, +VAL_INF, depth, best_move );
+        (*node_limit)--;
+      }
       else {
-        hsPutBOARD( tt_tag(param), node,
-                    (r == NOT_CHECK_MATE ? +VAL_INF : +VAL_MAY_MATCH), 0,
-                    param.te_count, te_ret );
+        hsPutBOARD( tt_tag(param), node, 
+                    (r == NOT_CHECK_MATE ? +VAL_INF : +VAL_MAY_MATCH), 0, 
+                    depth, best_move );
+        (*node_limit)--;
       }
-
       return;
     }
   }
@@ -629,16 +646,11 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
     te_clear(&best_move);
 
     // ノードを展開して最善手を得る
-    expand_and_best_move( taboo_list, param, node,
-                          depth, *node_limit,
-                          &best_child,
-                          &child_phi_sum, &second_child_delta,
-                          &best_move );
-    // しきい値を超えた？
-    if ( best_child.delta >= threshold.phi || child_phi_sum >= threshold.delta )
-      break;
-
-    assert( te_to(best_move) ); // 候補手が一つでもあれば必ず設定される
+    mate_expand_and_best_move( taboo_list, param, node,
+                               depth, *node_limit,
+			       &best_child,
+			       &child_phi_sum, &second_child_delta,
+			       &best_move );
 
 #ifdef USE_GUI
     // 中断や時間切れが入ってないか
@@ -646,6 +658,12 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
       break;
 #endif // USE_GUI
 
+    // しきい値を超えた？
+    if ( best_child.delta >= threshold.phi || child_phi_sum >= threshold.delta )
+      break;
+
+    assert( te_to(best_move) ); // 候補手が一つでもあれば必ず設定される
+
     if ( !teCmpTE(best_move, prev_move) ) {
 #if 0
       printf( "%s: warning: search loop??? depth=%d, threshold=(%d,%d), best_child_delta=%d, child_phi_sum=%d\n",
@@ -657,7 +675,7 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
 #if DEBUGLEVEL >= 3
     if (param.start_tesuki >= DEBUG_TESUKI_LEVEL) {
       printf( "move %s (%d, %d)\n",
-              te_str(best_move, *node).c_str(), best_child.delta, 
+              te_str(best_move, *node).c_str(), best_child.delta,
               child_phi_sum );
     }
 #endif
@@ -722,7 +740,7 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
 
       // 詰んでたら、何で合いしても詰む -> 局面を逆算して置換表に登録
       int32_t p = 0; int32_t d = 0;
-      hsGetBOARD( tt_tag(param), tmp_bo, &p, &d, NULL, param.te_count );
+      hsGetBOARD( tt_tag(param), tmp_bo, &p, &d, NULL, depth );
       if ( p >= +VAL_INF && d == 0 )
         set_muda_ai( tt_tag(param), node, param.te_count );
 
@@ -734,15 +752,16 @@ static void bfs_hisshi_search( TabooList<PhiDelta>* taboo_list,
   if (param.start_tesuki >= DEBUG_TESUKI_LEVEL) {
     printf( "%d: put %" PRIx64 "-> (%d,%d), best = %s\n",
             depth, node->key, best_child.delta, child_phi_sum,
-            te_str(best_move, node).c_str() );
+            te_str(best_move, *node).c_str() );
   }
 #endif
 
+  taboo_list->remove(node);
+
   // ノードの証明数を更新
   hsPutBOARD( tt_tag(param), node, best_child.delta, child_phi_sum,
-              param.te_count, best_move );
+              depth, best_move );
   (*node_limit)--;
-  taboo_list->remove(node);
 }
 
 
@@ -756,10 +775,17 @@ MATE_STATE bfs_hisshi( BOARD* bo, int tesuki, int depth, TE* best_move,
   assert( tesuki >= 0 && tesuki <= 2 );
   assert( best_move );
   assert( node_limit );
+#ifndef NDEBUG
+  uint64_t orig_hash_all = bo->hash_all;
+  if (bo->hash_all == 0x70dc4d3f4c226341ULL) {
+    printBOARD(bo);
+    abort();
+  }
+#endif
 
   TabooList<PhiDelta> taboo_list;
   EndingSearchParam param;
-  int32_t phi, delta;
+  PhiDelta score;
   int r;
 
   param.offense_side = bo->next;
@@ -768,34 +794,35 @@ MATE_STATE bfs_hisshi( BOARD* bo, int tesuki, int depth, TE* best_move,
   param.start_tesuki = tesuki;
 
   // 内部ノードで詰みを探すときなど、すでに結果が分かっている場合がある
-  phi = 0; delta = 0;
-  r = hsGetBOARD( tt_tag(param), bo, &phi, &delta, best_move, 0 );
+  score.phi = 0; score.delta = 0;
+  r = hsGetBOARD( tt_tag(param), bo, &score.phi, &score.delta, best_move, depth );
   if (r) {
     // 結論が出ている場合のみ
-    if ( phi == 0 && delta >= VAL_INF )
+    if ( score.phi == 0 && score.delta >= VAL_INF )
       return CHECK_MATE;
-    else if ( phi >= VAL_INF && delta == 0 )
+    else if ( score.phi >= +VAL_INF && score.delta == 0 )
       return NOT_CHECK_MATE;
   }
 
   PhiDelta threshold = make_phi_delta( +VAL_MAY_MATCH, +VAL_MAY_MATCH );
-
   bfs_hisshi_search( &taboo_list, param, bo, tesuki, threshold,
                      depth, node_limit );
+  assert(orig_hash_all == bo->hash_all);
 
   if ( gmGetGameStatus(&g_game) != 0 )
     return UNKNOWN_CHECK_MATE;
 
-  phi = 0; delta = 0;
-  r = hsGetBOARD( tt_tag(param), bo, &phi, &delta, best_move, 0 );
+  r = hsGetBOARD( tt_tag(param), bo, &score.phi, &score.delta, best_move, depth );
   if (!r) {
-    si_abort("%s: internal error: not found result.\n", __func__);
+    printf("%s: internal error: result missing: depth=%d\n", __func__, depth);
+    printBOARD(bo);
+    abort();
   }
 
-  if ( phi == 0 && delta >= VAL_MAY_MATCH )
+  if ( score.phi == 0 || score.delta >= VAL_MAY_MATCH )
     return CHECK_MATE;
   else {
-    if ( phi >= +VAL_INF && delta == 0 )
+    if ( score.phi >= +VAL_INF && score.delta == 0 )
       return NOT_CHECK_MATE;
     else
       return UNKNOWN_CHECK_MATE;
