@@ -21,23 +21,29 @@
 
 #include <stdio.h>
 #include <assert.h>
+#include <signal.h> // sig_atomic_t
 #include "si.h"
 #include "si-think.h"
-#include "ui.h"
-#include "taboo_list.h"
 #include <algorithm>
 
-#define DEBUGLEVEL 0
+#ifdef NDEBUG
+  // リリースモード
+  #define DEBUGLEVEL 0
+#else
+  #define DEBUGLEVEL 0
+#endif // NDEBUG
 
 #if DEBUGLEVEL >= 3
-#define __STDC_FORMAT_MACROS 1
-#include <inttypes.h>
+  #define __STDC_FORMAT_MACROS 1
+  #include <inttypes.h>
 #endif
+
 using namespace std;
 
 
 #ifdef USE_GUI
 extern GAME g_game;
+extern volatile sig_atomic_t gui_pending_counter;
 #endif
 
 
@@ -54,18 +60,12 @@ static bool lookup_hash( BOARD* bo,
   if ( hsGetBOARDMinMax(bo, score, best_move, te_count) )
     return true;
 
-  think_eval(bo, score);
+  score->phi = +VAL_LEAF_UNKNOWN;
+  score->delta = -think_eval( bo );
   return false;
 }
 
 
-struct SearchParam
-{
-  int te_count;
-  int start_depth;
-};
-
-
 /** 置換表に勝ちを登録 */
 struct PutWin {
   int te_count;
@@ -77,24 +77,25 @@ struct PutWin {
 
 
 /** 
- * 手を指す. 悪手のときは指さない
+ * 手を指して、指した後の（仮の）評価値を child_score に格納する.
+ * 悪手のときは指さない
  * @return 悪手のとき, 指さずに false を返す
  */
 static bool do_move( const SearchParam& param,
                      int rest_depth,
                      BOARD* node,
 		     const TsumeroNogare& tsumero_nogare,
-                     MOVEINFO* const mi, int index,
+                     TE* te,
 		     PhiDelta* child_score )
 {
-  if ( (te_hint(mi->te[index]) & TE_STUPID) != 0 )
+  if ( (te_hint(*te) & TE_STUPID) != 0 )
     return false;
 
   bool is_checked = bo_is_checked(node, node->next);
   int atkcnt = think_count_attack24(node, node->next, 
 				    node->king_xy[1 - node->next]);
 
-  boMove( node, mi->te[index] );
+  boMove( node, *te );
 
   // 千日手？ とりあえず1回目でもループは避ける
   if ( bo_repetition_count(node) >= 1 ) {
@@ -107,8 +108,8 @@ static bool do_move( const SearchParam& param,
                         &tt_move, 
 			child_score );
   if (r) {
-    if ( !teCmpTE(mi->te[index], tt_move) )
-      te_set_hint( &mi->te[index], te_hint(tt_move) );
+    if ( !teCmpTE(*te, tt_move) )
+      te_set_hint( te, te_hint(tt_move) );
 
     return true;
   }
@@ -122,7 +123,8 @@ static bool do_move( const SearchParam& param,
     return true;
   }
 
-  if ( (te_hint(node->mseq.te[node->mseq.count - 2]) & TE_TSUMERO) != 0) {
+  if ( node->mseq.count >= 2 && 
+       (te_hint(node->mseq.te[node->mseq.count - 2]) & TE_TSUMERO) != 0) {
     // 詰めろを掛けられていたとき
     if ( !tsumero_nogare.promise(node) ) {
       // 判定が軽いときは置換表に登録するまでもない
@@ -137,12 +139,13 @@ static bool do_move( const SearchParam& param,
     if ( dfpn_mate(node, 31, &mate_te, &mate_node_limit) == CHECK_MATE ) {
       // 受けになっていない. 負け
       hsPutBOARDMinMax(node, +VAL_INF, -VAL_INF, param.te_count, TE_NUL );
-
-      if ( te_is_put(mi->te[index]) ) {
+/*
+      if ( te_is_put(*te) ) {
         // ほかの打つ手はどうか
         mate_set_muda_uchi( node->next, 1, PutWin(param.te_count),
                             node, mi, index + 1 );
       }
+*/
       boBack(node);
       return false;
     }
@@ -150,17 +153,21 @@ static bool do_move( const SearchParam& param,
 
   // こちらから詰めろ (または詰めろ逃れの詰めろ) が掛かるか
   // ルートノードのときのみ調べる. TODO: より深いノードでも調べる
-  if ( param.start_depth == rest_depth ) {
+  if ( param.start_depth <= rest_depth ) {
     bool promise = false;
+    int atkcnt2;
 
     // - 取る手
     if ( node->tori[node->mseq.count - 1] )
       promise = true;
 
     // - 利きを入れる手
-    else if ( think_count_attack24(node, 1 - node->next, 
-				 node->king_xy[node->next]) > atkcnt )
-      promise = true;
+    else {
+      atkcnt2 = think_count_attack24( node, 1 - node->next, 
+				      node->king_xy[node->next] );
+      if ( atkcnt2 > atkcnt )
+	promise = true;
+    }
 
     if ( promise ) {
       // 詰めろ？
@@ -170,77 +177,97 @@ static bool do_move( const SearchParam& param,
       if ( dfpn_mate(node, 31, &mate_te, &mate_node_limit) == CHECK_MATE ) {
         // 詰めろを掛けたときは仮評価を足して、優先的に読むように
 	// child_score->phi -= 1000 * 100;
-        child_score->delta += 1000 * 100; // TODO: NPC へ移動
-        te_add_hint( &mi->te[index], TE_TSUMERO );
+        child_score->delta += 2000 * 100; // TODO: magic. NPC へ移動
+        te_add_hint( te, TE_TSUMERO );
+      }
+      else {
+	// 2手スキが掛かるか
+	// TODO: 再起で書いた方がいい
+	MOVEINFO mi2;
+	make_moves(node, &mi2);
+	for (int i2 = 0; i2 < mi2.count; i2++) {
+	  boMove(node, mi2.te[i2]); // 2手目
+	  bool promise2 = false;
+	  if ( node->tori[node->mseq.count - 1] )
+	    promise2 = true;
+	  else if ( think_count_attack24(node, 1 - node->next,
+					 node->king_xy[node->next]) > atkcnt2 )
+	    promise2 = true;
+
+	  if ( promise2 ) {
+	    bo_pass(node);
+	    int mate_node_limit2 = GAME_MATE_NODE_LIMIT;
+	    TE mate_te2;
+	    if ( dfpn_mate(node, 31, &mate_te2, &mate_node_limit2) == 
+		    CHECK_MATE ) {
+	      child_score->delta += 1900 * 100; // TODO: magic. NPCへ移動
+	      te_add_hint( te, TE_2TESUKI );
+
+	      boBack_mate(node);
+	      boBack(node);
+	      break;
+	    }
+	    boBack_mate(node); // パスを戻す
+	  }
+	  boBack(node);
+
+#ifdef USE_GUI
+	  // 中断や時間切れが入ってないか
+	  if ( gmGetGameStatus(&g_game) != 0 )
+	    break;
+#endif // USE_GUI
+	} // for
       }
       boBack_mate(node); // パスを戻す
       return true;
     }
-
-    // 2手指してみる
-    bo_pass( node );
-
-    MOVEINFO mi;
-    make_moves( node, &mi );
-
-    child_score->phi = +VAL_INF; child_score->delta = -VAL_INF;
-    for (int i = 0; i < mi.count; i++) {
-      boMove( node, mi.te[i] );
-
-      PhiDelta c;
-      think_eval( node, &c );
-      if ( (c.delta - c.phi) > (child_score->delta - child_score->phi) ) {
-	child_score->phi = c.phi; child_score->delta = c.delta;
-      }
-
-      boBack( node );
-    }
-    boBack_mate( node ); // パスを戻す
   }
 
   return true;
 }
 
 
+#if DEBUGLEVEL >= 3
+static string score_str_val(int32_t v) 
+{
+  if (v == +VAL_INF)
+    return "INF";
+  else if (v == +VAL_LEAF_UNKNOWN)
+    return "UNK";
+  else if (v == -VAL_LEAF_UNKNOWN)
+    return "-UNK";
+  else if (v == -VAL_INF)
+    return "-INF";
+  else
+    return str_format("%d", v);
+}
+
+static string score_str(const PhiDelta& s)
+{
+  return "(" + score_str_val(s.phi) + "," + score_str_val(s.delta) + ")";
+}
+#endif // !NDEBUG
+
+
 /**
- * 手を展開し、最善手を選ぶ
- * 子ノードのうち delta 最小を選ぶ
+ * 手を展開し、仮の評価値を得る
+ * 子ノードのうち delta 最大 (子ノードにとって悪い手) を選ぶ
  * 勝ち = (+VAL_INF, -VAL_INF), 負け = (-VAL_INF, +VAL_INF)
  * 一通り展開するので、move ordering は効果なし
  */
-static void expand_and_best_move( const SearchParam& param,
+void search_expand_trial_values( const SearchParam& param,
                                   BOARD* node,
                                   int rest_depth, int rest_node,
-                                  PhiDelta* best_child,
-				  // int32_t* child_phi_min,
-                                  PhiDelta* second_child,
-                                  TE* best_move )
+				  RankSet* rank )
 {
-  // 手を展開する
   MOVEINFO mi;
   make_moves(node, &mi);
 
-  best_child->delta = -VAL_INF; best_child->phi = +VAL_INF; // worst value
-  // *child_phi_min = +VAL_INF;
-
   if (!mi.count) {
-    // 打ち歩詰め. ノード側の勝ち
-    if ( node->mseq.count >= 1 && 
-	 te_is_put(node->mseq.te[node->mseq.count - 1]) &&
-	 te_pie(node->mseq.te[node->mseq.count - 1]) == FU ) {
-      best_child->phi = -VAL_INF; best_child->delta = +VAL_INF;
-      te_clear(best_move);
-      return;
-    }
-
     // 負け
-    te_clear(best_move);
     return;
   }
 
-  *second_child = *best_child;
-  const TE* may_best = mi.te; // 手がある限り何か選ぶ
-
 #if DEBUGLEVEL >= 3
   printf("depth %d: ", rest_depth);
 #endif
@@ -250,59 +277,55 @@ static void expand_and_best_move( const SearchParam& param,
   tsumero_nogare.before_move( node, 1 - node->next );
 
   for (int i = 0; i < mi.count; i++) {
-    PhiDelta child_score;  // child nodeにとってよい局面 -> phi大, delta小
+    MoveScore move_score;   // child nodeにとってよい局面 -> phi大, delta小
 
     if ( !do_move(param, rest_depth, node, tsumero_nogare,
-		  &mi, i, &child_score) )
+		  &mi.te[i], &move_score.score) ) {
       continue;
-
-    if ( (child_score.delta - child_score.phi) > 
-	 (best_child->delta - best_child->phi) ) {
-      // bestを更新
-      *second_child = *best_child;
-      *best_child = child_score;
-      may_best = mi.te + i;
-
-      // 選択した手のphi
-      // *child_phi_min = child_score.phi;
-    }
-    else if ( (child_score.delta - child_score.phi) > 
-	      (second_child->delta - second_child->phi) ) {
-      // 2位のみ更新
-      *second_child = child_score;
     }
 
-    // TODO: これでいいかどうか -> 選択しなかった手の値になることがある. 不味そう
-    // *child_phi_min = min(*child_phi_min, child_score.phi);
+    // ソートしながら整理
+    move_score.te = mi.te[i];
+    rank->insert(move_score);
 
     boBack(node);
 
 #if DEBUGLEVEL >= 3
-    printf("%s=(%d,%d), ", te_str(mi.te[i], *node).c_str(),
-           child_score.phi, child_score.delta);
+    printf( "%s=%s, ", 
+       te_str(mi.te[i], *node).c_str(), score_str(move_score.score).c_str() );
 #endif
 
-    if ( child_score.phi <= -VAL_MAY_MATCH || 
-         child_score.delta >= +VAL_MAY_MATCH ) {
-      break;  // nodeの手番側の勝ち確定
+    // nodeの手番側の勝ち確定
+    if ( move_score.score.phi <= -VAL_INF || 
+                               move_score.score.delta >= +VAL_INF ) {
+      break;  
     }
   }
-
 #if DEBUGLEVEL >= 3
-  printf("best = %s\n", te_str(*may_best, *node).c_str());
+  printf("\n");
+//   printf("best = %s\n", te_str(*may_best, *node).c_str());
 #endif
-  *best_move = *may_best;
+}
+
+
+static int32_t new_delta(int32_t node_delta, int32_t child_phi) 
+{
+  if ( child_phi != +VAL_LEAF_UNKNOWN )
+    return child_phi;
+  else
+    return node_delta;
 }
 
 
 /**
- * 探索する
+ * 探索し、現局面き評価値を更新する。
  * 必ず一度はノードを展開する
  */
 static void bfs_search( const SearchParam& param,
-			BOARD* node,
+			BOARD* node, 
                         const PhiDelta& threshold,
-                        int depth, int* node_limit )
+                        int depth, int* node_limit, 
+			PhiDelta* node_score, TE* best_move )
 {
 #ifndef NDEBUG
   uint64_t const node_hash = node->hash_all;
@@ -313,67 +336,59 @@ static void bfs_search( const SearchParam& param,
          depth, threshold.phi, threshold.delta, *node_limit);
 #endif
 
-  TE best_move, prev_move;
+  TE prev_move;
   PhiDelta prev_threshold = make_phi_delta(0, 0);
-  te_clear(&best_move);
+  te_clear(best_move);
   te_clear(&prev_move);
 
 #ifdef USE_GUI
-  if ( !(*node_limit % 700) ) {
-    /* GUIのペンディングされたイベントを処理する */
+  if ( gui_pending_counter ) {
     processing_pending( false );
+    gui_pending_counter = 0;
   }
 #endif /* USE_GUI */
 
-  if ( param.start_depth == depth ||
-       (node->mseq.count >= 1 && 
-        (te_hint(node->mseq.te[node->mseq.count - 1]) & TE_TSUMERO) != 0) ) {
-    // ルートノードか、詰めろを掛けられていた. 勝ちがあるか
-    int mate_node_limit = GAME_MATE_NODE_LIMIT;
-    if ( dfpn_mate(node, 31, &best_move, &mate_node_limit) == CHECK_MATE ) {
-      hsPutBOARDMinMax( node, +VAL_INF, -VAL_INF, param.te_count, best_move );
-      (*node_limit)--;
-      return;
-    }
-  }
+  RankSet rank;
+  RankSet::const_iterator it;
 
-  if ( node->mseq.count >= 2 && 
-       (te_hint(node->mseq.te[node->mseq.count - 2]) & TE_TSUMERO) != 0 ) {
-    // 詰めろを掛けた次の手で、詰め将棋を読んでみる
+  // 次の場合に、詰みを読む。
+  // (1) ルートノードか、頓死チェック
+  // (2) 詰めろを掛けられていた. 
+  // (3) 詰めろを掛けた次の手
+  if ( param.start_depth - depth <= 1 ||
+       (node->mseq.count >= 1 && 
+        (te_hint(node->mseq.te[node->mseq.count - 1]) & TE_TSUMERO) != 0) ||
+       (node->mseq.count >= 2 && 
+	(te_hint(node->mseq.te[node->mseq.count - 2]) & TE_TSUMERO) != 0) ) {
     int mate_node_limit = GAME_MATE_NODE_LIMIT;
-    if ( dfpn_mate(node, 31, &best_move, &mate_node_limit) == CHECK_MATE ) {
-      hsPutBOARDMinMax( node, +VAL_INF, -VAL_INF, param.te_count, best_move );
-      (*node_limit)--;
-      return;
+    if ( dfpn_mate(node, 31, best_move, &mate_node_limit) == CHECK_MATE ) {
+      node_score->phi = +VAL_INF; node_score->delta = -VAL_INF;
+      goto exit_search;
     }
   }
 
   // サイクル回避
   bo_inc_repetition(node);
 
-  PhiDelta best_child;
-  // int32_t child_phi_min;
+  // 1回目はノードを展開して (仮の) 最善手を得る.
+  search_expand_trial_values( param, node, /* mi,*/
+			depth, *node_limit,
+			&rank );
+  assert( node_hash == node->hash_all );
 
   while (true) {
-    PhiDelta second_child;
-
-    te_clear(&best_move);
-
-    // ノードを展開して最善手を得る
-    expand_and_best_move( param, node,
-                          depth, *node_limit,
-                          &best_child,
-			  // &child_phi_min,
-                          &second_child,
-                          &best_move );
-
-    // 閾値を超えた？
-    if ( (best_child.delta - best_child.phi) <= threshold.phi || 
-	 -(best_child.delta - best_child.phi) <= threshold.delta )
+    // 手がない
+    if ( rank.size() == 0 ) {
+      te_clear(best_move);
       break;
+    }
 
-    // 候補手が一つでもあれば必ず設定されるはず
-    assert( te_to(best_move) );
+    // 現在の最善手
+    it = rank.begin();
+    *best_move = it->te;
+#if DEBUGLEVEL >= 3
+    printf("best = %s\n", te_str(it->te, *node).c_str());
+#endif
 
 #ifdef USE_GUI
     /* 中断や時間切れが入ってないか調べる */
@@ -385,17 +400,27 @@ static void bfs_search( const SearchParam& param,
     if ( depth <= 1 || *node_limit < 0 )
       break;
 
+    // 閾値を超えた？
+    if ( it->score.delta <= threshold.phi || it->score.phi <= threshold.delta )
+      break;
+
+    // ノードを展開する
     PhiDelta threshold_child;
 
-    // このしきい値を超える (下回る) -> delta cutが起こる
+    // このしきい値を超える (下回る) -> このノードに戻ったときにdelta cutが起こる
     threshold_child.phi = threshold.delta;
 
     // このしきい値を超える (下回る) -> 2位の方が有望
-    threshold_child.delta = max( threshold.phi, 
-				 (second_child.delta - second_child.phi) - 1 );
+    if (rank.size() == 1)
+      threshold_child.delta = threshold.phi;
+    else {
+      RankSet::const_iterator it2 = it;
+      ++it2;
+      threshold_child.delta = max( threshold.phi, it2->score.delta - 1 );
+    }
 
     // 先端 ?
-    if ( !teCmpTE(prev_move, best_move) && prev_threshold == threshold_child ) {
+    if ( !teCmpTE(prev_move, it->te) && prev_threshold == threshold_child ) {
 #if 0
       printf( "%d: loop!! threshold=(%d, %d), score=(%d,%d)\n",
 	      depth, threshold.phi, threshold.delta, 
@@ -404,30 +429,48 @@ static void bfs_search( const SearchParam& param,
       break;
     }
 
-    boMove(node, best_move);
-
-    bfs_search( param, node, threshold_child,
-                depth - 1, node_limit );
+    MoveScore new_move_score;
+    new_move_score.score = it->score;
+    TE te_dmy;
 
+    // TODO: ここを並列化できる
+    boMove(node, it->te );
+    bfs_search( param, node, threshold_child, depth - 1, 
+                node_limit, 
+		&new_move_score.score, &te_dmy );
     boBack(node);
 
-    prev_move = best_move;
+    prev_move = it->te;
     prev_threshold = threshold_child;
+
+    // 読んだ手（とその得点）を差し替える
+    new_move_score.te = it->te;
+    rank.erase(it);
+    rank.insert(new_move_score);
   }
 
   assert( node_hash == node->hash_all );
-  hsPutBOARDMinMax( node, best_child.delta, best_child.phi /*child_phi_min*/, 
-		    param.te_count, best_move );
 
-#if DEBUGLEVEL >= 3
-    printf("%d: put %" PRIx64 "-> (%d,%d)\n",
-           depth, node->key, best_child.delta, child_phi_min );
-#endif
-
-  (*node_limit)--;
+  if (rank.size() == 0) {
+    node_score->phi = -VAL_INF; node_score->delta = +VAL_INF;
+  }
+  else {
+    node_score->phi = it->score.delta;
+    node_score->delta = new_delta(node_score->delta, it->score.phi);
+  }
 
   // タブーリストから除く
   bo_dec_repetition(node);
+
+exit_search:
+  hsPutBOARDMinMax( node, node_score->phi, node_score->delta, param.te_count,
+		    *best_move );
+#if DEBUGLEVEL >= 3
+    printf("%d: put %" PRIx64 "-> %s %s\n",
+	   depth, node->key, 
+	   te_str(*best_move, *node).c_str(), score_str(*node_score).c_str() );
+#endif
+  (*node_limit)--;
 }
 
 
@@ -439,45 +482,72 @@ static void bfs_search( const SearchParam& param,
  *
  * @return SI_NORMAL
  */
-INPUTSTATUS bfs_minmax( BOARD* bo, int depth, TE* best_move )
+INPUTSTATUS bfs_minmax( BOARD* bo, int depth, PhiDelta* score, TE* best_move )
 {
+  assert(score);
   assert(best_move);
 
   int r;
-  PhiDelta score;
+  // PhiDelta score;
 
   // 結論が出ているか？
-  r = hsGetBOARDMinMax( bo, &score, best_move, 0 );
+  r = hsGetBOARDMinMax( bo, score, best_move, 0 );
   if (r) {
-    // 結論が出ている場合のみ利用
-    if ( score.phi >= +VAL_INF && score.delta <= -VAL_INF ) {
+    // 勝ち・負けの場合のみ利用
+    if ( score->phi >= +VAL_INF || score->delta <= -VAL_INF ) {
       assert( te_to(*best_move) >= 0x11 && te_to(*best_move) <= 0x99);
       return SI_NORMAL;
     }
-    else if ( score.phi <= -VAL_INF && score.delta >= +VAL_INF ) {
+    else if ( score->phi <= -VAL_INF || score->delta >= +VAL_INF ) {
       te_clear(best_move);
       return SI_NORMAL;
     }
   }
 
+  bool checked = bo_is_checked(bo, bo->next);
+#ifndef NDEBUG
+  if (checked && bo->mseq.count >= 1) {
+    assert( (te_hint(bo->mseq.te[bo->mseq.count - 1]) & TE_CHECK) != 0 );
+  }
+#endif
+
+  // 詰めろを掛けられているか？
+  if ( !checked && bo->mseq.count >= 1 &&
+       !(te_hint(bo->mseq.te[bo->mseq.count - 1]) & TE_TSUMERO) ) {
+    bo_pass(bo);
+    int node_limit = GAME_MATE_NODE_LIMIT;
+    TE te_ret;
+    if ( dfpn_mate(bo, 31, &te_ret, &node_limit) == CHECK_MATE ) {
+      // 詰めろ
+      te_add_hint(&bo->mseq.te[bo->mseq.count - 2], TE_TSUMERO);
+    }
+    boBack_mate(bo);
+  }
+  // 中断, 時間切れ?
+  if ( gmGetGameStatus(&g_game) != 0 )
+    return gmGetGameStatus(&g_game);
+
   SearchParam param;
   param.te_count = bo->mseq.count;
   param.start_depth = depth;
   int node_limit = 25000; // magic
+  *score = make_phi_delta(+VAL_LEAF_UNKNOWN, +VAL_LEAF_UNKNOWN);
   bfs_search( param, bo, 
-	      make_phi_delta(-VAL_MAY_MATCH, -VAL_MAY_MATCH),
-              depth, &node_limit );
+	      make_phi_delta(-VAL_INF, -VAL_INF),
+              depth, &node_limit, score, best_move );
 
   // 中断または時間切れ
   if ( gmGetGameStatus(&g_game) != 0 )
     return gmGetGameStatus(&g_game);
 
-  r = hsGetBOARDMinMax( bo, &score, best_move, 0 );
+#if 0
+  r = hsGetBOARDMinMax( bo, score, best_move, 0 );
   if (!r) {
     si_abort("internal error: not found result\n");
   }
+#endif // 0
 
-  if ( score.phi <= -VAL_MAY_MATCH && score.delta >= +VAL_MAY_MATCH ) {
+  if ( score->phi <= -VAL_INF || score->delta >= +VAL_INF ) {
     te_clear(best_move);
     return SI_NORMAL;
   }
