@@ -23,7 +23,6 @@
 #include <assert.h>
 #include <algorithm>
 #include "si.h"
-#include "ui.h"
 using namespace std;
 
 
@@ -38,7 +37,7 @@ static const int can_ohte_to[8] = {
  * @param mi 結果を格納するMOVEINFO
  * @param king_xy 受け方の玉の位置
  */
-void normalOhte(const BOARD* bo, MOVEINFO* mi, Xy king_xy) 
+static void normalOhte(const BOARD* bo, MOVEINFO* mi, Xy king_xy) 
 {
   static const int can_ohte[8][32][2] = {
       {
@@ -300,6 +299,94 @@ static void remoteOhte(const BOARD* bo, MOVEINFO* mi, Xy const king_xy)
 }
 
 
+/**
+ * ある駒を打つ手を生成.
+ * utiOhte() から呼び出される. 
+ * 二歩, 打ち歩詰めになる手は生成しない
+ *
+ * @param bo 局面
+ * @param mi 追加するMOVEINFO
+ * @param pie 駒種
+ * @param xy 座標. 0x11-0x99まで
+ */
+static void mi_append_a_put_move( const BOARD* bo, MOVEINFO* mi, 
+                                  PieceKind pie, Xy xy )
+{
+  TE te = te_make_put(xy, pie);
+
+  if (bo->next == 0) {
+    // 先手
+    switch (pie) {
+    case FU:
+      if (checkFuSente(bo, xy & 0xf) || xy <= 0x19)
+        return;
+
+      // 打ち歩詰め？
+      if (bo->board[xy - 16] == (OH + 0x10)) {
+        if (te_is_uchifuzume(bo, te))
+          return;
+      }
+      break;
+    case KYO:
+      if (xy <= 0x19)
+        return;
+      break;
+    case KEI:
+      if (xy <= 0x29)
+        return;
+      break;
+    default:
+      break;
+    }
+  }
+  else {
+    // 後手
+    switch (pie) {
+    case FU:
+      if (checkFuGote(bo, xy & 0xf) || xy >= 0x91)
+        return;
+
+      if (bo->board[xy + 16] == OH) {
+        if (te_is_uchifuzume(bo, te))
+          return;
+      }
+      break;
+    case KYO:
+      if (xy >= 0x91)
+        return;
+      break;
+    case KEI:
+      if (xy >= 0x81)
+        return;
+      break;
+    default:
+      break;
+    }
+  }
+
+  miAdd(mi, te);
+}
+
+
+/**
+ * ある場所に打つ手をすべて生成
+ * 2009.11 ループの内部でこの関数を呼び出すと遅い. 
+ */
+void mi_append_put_moves(const BOARD* bo, Xy xy, MOVEINFO* mi)
+{
+  assert( xy >= 0x11 && xy <= 0x99 );
+  assert( !bo->board[xy] );
+
+  int i;
+  for (i = 1; i <= 7; i++) {
+    if ( !bo->inhand[bo->next][i] )
+      continue;
+
+    mi_append_a_put_move(bo, mi, i, xy);
+  }
+}
+
+
 /**
  * 歩、桂、銀、金の打ち込んで王手を探す。
  * @param bo 対象のBOARD
@@ -318,7 +405,7 @@ static void utiOhte(const BOARD* bo, MOVEINFO* mi, Xy const king_xy)
       if (bo->board[to]) // 盤外か駒がある
         continue;
 
-      append_a_put_move(bo, mi, pie, to);
+      mi_append_a_put_move(bo, mi, pie, to);
     }
   }
 }
@@ -424,6 +511,163 @@ static bool can_attack_to( PieceKind const p, const BOARD& bo,
 }
 
 
+/**
+ * 長い攻撃の指し手を生成する
+ * @param mi 結果を格納するMOVEINFO
+ * @param bo 対象のBOARD
+ * @param xy 駒の座標
+ * @param next 駒の持ち主
+ */
+static void movetoLong(MOVEINFO* mi, const BOARD* bo, Xy const xy, int next)
+{
+  assert( 1 <= (xy & 0xf) && (xy & 0xf) <= 9 );
+  assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
+  assert( bo->board[xy] != 0 );
+  assert( getTeban(bo->board[xy]) == bo->next );
+
+  int i;
+  PieceKind piece = boGetPiece(bo, xy);
+  // PieceKind p = piece & 0x0F;
+
+  for (i = 0; i < remote_to_count[piece]; i++) {
+#ifdef USE_PIN
+    // こちらでもpinされていないか確認しておく
+    if (bo->pinned[xy]) {
+      if (bo->pinned[xy] != remote_to[piece][i] &&
+           bo->pinned[xy] != -remote_to[piece][i])
+        continue;
+    }
+#endif
+
+    bool flag = false;
+    Xy vw = xy;
+    while (1) {
+      vw += remote_to[piece][i];
+
+      PieceKind const c = boGetPiece(bo, vw);
+      if (c == WALL)
+        break;
+
+      if (c) {
+        // 自分の駒があるとダメ
+	if ( getTeban(c) == next )
+	  break;
+        flag = true;
+      }
+
+      // te.to = vw;
+      miAddWithNari( mi, *bo, xy, vw );
+
+      if (flag)
+        break;
+    }
+  }
+}
+
+
+/**
+ * xy にある駒の動ける場所 (指し手) を探してmiに追加する。
+ * @param bo 対象のBOARD
+ * @param xy 駒の場所
+ * @param mi 結果を格納するMOVEINFO
+ */
+static void mi_append_moveto(const BOARD* bo, Xy xy, MOVEINFO* mi)
+{
+  assert( 1 <= (xy & 0x0f) && (xy & 0x0f) <= 9 );
+  assert( 1 <= (xy >> 4) && (xy >> 4) <= 9 );
+  assert( bo->board[xy] != 0 );
+  assert( getTeban(bo->board[xy]) == bo->next );
+
+  PieceKind const piece = bo->board[ xy ];
+#if 0
+  if ( (piece & 0xf) == OH )
+    mi_append_king_moves( bo, mi, xy, bo->next );
+  else
+    append_short_onboard_moves( bo, xy, piece, mi );
+
+  if ( is_long_piece(piece) )
+    movetoLong(mi, bo, xy, bo->next);
+#else
+  // 展開してみる   2009.12 多少だが効果あり
+  switch (piece) {
+  case FU:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    break;
+  case KYO: case KYO + 0x10:
+  case KAKU: case KAKU + 0x10:
+  case HISHA: case HISHA + 0x10:
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case KEI:
+    miAddWithNari( mi, *bo, xy, xy - 33 );
+    miAddWithNari( mi, *bo, xy, xy - 31 );
+    break;
+  case GIN:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    break;
+  case KIN:
+  case TOKIN: case NARIKYO: case NARIKEI: case NARIGIN:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    break;
+  case OH:
+  case OH + 0x10:
+    mi_append_king_moves( bo, mi, xy, bo->next );
+    break;
+  case UMA:
+  case UMA + 0x10:
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case RYU:
+  case RYU + 0x10:
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    movetoLong( mi, bo, xy, bo->next );
+    break;
+  case FU + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    break;
+  case KEI + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 33 );
+    miAddWithNari( mi, *bo, xy, xy + 31 );
+    break;
+  case GIN + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy - 15 );
+    miAddWithNari( mi, *bo, xy, xy - 17 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    break;
+  case KIN + 0x10:
+  case TOKIN + 0x10: case NARIKYO + 0x10: case NARIKEI + 0x10: case NARIGIN + 0x10:
+    miAddWithNari( mi, *bo, xy, xy + 16 );
+    miAddWithNari( mi, *bo, xy, xy + 17 );
+    miAddWithNari( mi, *bo, xy, xy + 1 );
+    miAddWithNari( mi, *bo, xy, xy - 16 );
+    miAddWithNari( mi, *bo, xy, xy - 1 );
+    miAddWithNari( mi, *bo, xy, xy + 15 );
+    break;
+  default:
+    abort();
+  }
+#endif
+}
+
+
 /**
  * 空き王手を探す。
  * @param bo 対象のBOARD
@@ -554,3 +798,47 @@ void ohte(const BOARD* bo, MOVEINFO* mi)
   // 2009.12 これはだいぶ遅い
   // miRemoveDuplicationTE(mi); // 移動の手がダブる
 }
+
+
+/**
+ * 局面で着手可能なすべての指し手を生成する
+ * 打ち歩詰めの手は生成しない.
+ * 千日手になる手は生成する
+ *
+ * @param bo 局面
+ * @param mi 手一覧
+ */
+void make_moves(const BOARD* bo, MOVEINFO* mi)
+{
+  int i;
+  PieceKind inhand_pie[8];
+
+  // 王手を掛けられているか
+  if ( bo_is_checked(bo, bo->next) ) {
+    uke( bo, mi );
+    return;
+  }
+
+  mi->count = 0;
+
+  // あらかじめ持ち駒セットを作る. 2009.11 効果あり
+  int p = 0;
+  for (i = 1; i <= 7; i++) {
+    if ( bo->inhand[bo->next][i] > 0 )
+      inhand_pie[p++] = i;
+  }
+  inhand_pie[p] = 0;
+
+  for (i = 1; i <= 81; i++) {
+    if ( bo->board[arr_xy[i]] ) {
+      // xyから移動する手
+      if ( getTeban(bo->board[arr_xy[i]]) == bo->next ) 
+	mi_append_moveto(bo, arr_xy[i], mi);
+    }
+    else {
+      // 打つ手
+      for (p = 0; inhand_pie[p]; p++)
+	mi_append_a_put_move( bo, mi, inhand_pie[p], arr_xy[i] );
+    }
+  }
+}
