@@ -40,6 +40,7 @@ enum TEBAN {
   GOTE  = 1
 };
 
+extern const char* CSA_PIECE_NAME[];
 
 /** mate() が返すステータス */
 enum MATE_STATE {
@@ -63,35 +64,17 @@ enum INPUTSTATUS
   /** 通常の手, ローカルからの投了 */
   SI_NORMAL = 0,
 
-  /** 手番のプレイヤーが勝ち(相手が反則手を指した) */
-  // SI_WIN = 1, 
-
-  /** 手番のプレイヤーが投了 */
-  // SI_TORYO = 2,
-
-  /** 手番のプレイヤーが待った */
-  // SI_MATTA = 3,
-
   /** 手番のプレイヤーがゲームを中断 */
   SI_CHUDAN_LOCAL = 4,
 
   /** サーバーからの中断指示の受信 */
   SI_CHUDAN_REMOTE = 5,
 
-  /** 手番のプレイヤーが入玉で勝ちを宣言 */
-  // SI_KACHI = 6,
-
-  /** 手番のプレイヤーが入玉で引き分けを宣言 */
-  // SI_HIKIWAKE = 7,
-
-  /** 送信されてきた手が不正な手だった */
-  // SI_NGDATA = 8,
-
   /** 手番のプレイヤーが時間切れ負け */
   SI_TIMEOUT = 9,
 
-  /** 千日手で引き分け */
-  // SI_SENNICHITE = 10,
+  /** 直ちにプロセスを終了させる */
+  SI_TO_EXIT = 10,
 };
 
 
@@ -109,12 +92,7 @@ enum PLAYERTYPE {
   SI_COMPUTER_4 = 4,
   /** プログラムレベル５による入力 */
   SI_COMPUTER_5 = 5,
-#ifdef USE_SERIAL
-  /** シリアルポートからの入力１ */
-  SI_SERIALPORT_1 = 6,
-  /** シリアルポートからの入力２ */
-  SI_SERIALPORT_2 = 7,
-#endif
+
   /** ネットワークからの入力１ */
   SI_NETWORK_1 = 8,
   /** 子プロセスとパイプで通信 */
@@ -124,19 +102,6 @@ enum PLAYERTYPE {
 };
 
 
-/** 可能な合法手の最大は593 */
-#define MOVEINFOMAX 593
-
-/** 指し手 (候補手) を格納する構造体 */
-struct MOVEINFO 
-{
-  /** 指し手 */
-  TE te[MOVEINFOMAX + 1];
-  /** 指し手の数 */
-  int count;
-};
-
-
 #define MOVEINFO2MAX 2000
 
 /** 指し手 (手順) を格納する構造体. */
@@ -249,62 +214,18 @@ struct BOARD
 
   //////////////////////////////////////////////////////////////
   // 思考用
-
+#if EVAL_VERSION == 1
   /** 
    * 局面の評価点数. boMove() で更新できるものだけ. 
    * この変数を直接使わずに、think_eval() で評価を得ること 
    */
   int32_t point[2];
-
-  /** 消費時間の合計. \todo GAMEへ移動 */
-  time_t total_time[2];
-
-  /** 制限時間. \todo GAMEへ移動 */
-  time_t limit_time[2];
-
-  /** 思考開始前の時刻. \todo GAMEへ移動 */
-  time_t before_think_time;
+#endif // EVAL_VERSION == 1
 };
 
 
-/** 置換表のエントリ. 持ち駒も区別する. PODでなければならない */
-struct EntVariant 
-{
-  /** 
-   * 持駒 (packed). ビットの数で表すわけではないので、
-   * 計算できるのは一致のみ. 
-   */
-  uint32_t packed_inhand[2];
-
-  /** 得点 */
-  int32_t phi, delta;
-
-  /** 記録したときの手数 (深さではない) */
-  int32_t te_count;
-
-  /** 最善手 */
-  uint32_t best_move;
-};
-#define HASH_ENT_SIZE 24 // x86-64: 8の倍数
-
-
-/** 盤面を記録する置換表のエントリ. PODでなければならない */
-struct HASH
-{
-  /** 
-   * 持ち駒を含まないハッシュ値.
-   * \todo 削除する？ 
-   */
-  uint64_t key;
-
-  /** 得点. 複数記録する */
-  struct EntVariant ent[1];
-};
-#define HASH_HEADER_SIZE 8 // ent[]は除く
-
-
-///////////////////////////////////////////////////////////////////////
-// 定跡
+////////////////////////////////////////////////////////////////////////
+// 定跡 book.cc
 
 /** 定跡エントリ */
 struct BookEnt {
@@ -324,33 +245,41 @@ BookEnt* book_get( uint64_t hash_all, int* count );
 
 void book_put( uint64_t hash_all, const BookEnt* ent, int count );
 
+int jouseki(const BOARD* bo, TE* te);
+
+
+
+////////////////////////////////////////////////////////////////////////
+// GAME
 
-///////////////////////////////////////////////////////////////////////
 
 typedef INPUTSTATUS (*InputCallbackFunc)( const BOARD* bo, 
 					  DTe* te, int* elapsed_time );
 
-/** ゲームの進行に必要な情報を集めた構造体 */
+/** 
+ * ゲームの進行に必要な情報を集めた構造体
+ * Recordに格納するものは除く
+ */
 struct GAME
 {
-  /** 手合 */
-  // TEAI teai;
-
-  /** player type */
-  // PLAYERTYPE player_number[2];
-
   /** 入力を受ける関数へのポインタ */
   InputCallbackFunc si_input_next[2];
 
   /** コンピュータの思考レベル */
   int computer_level[2];
-#ifdef USE_SERIAL
-  /** シリアルポートを使い通信対局するのための構造体 */
-  COMACCESS ca[2];
-#endif /* USE_SERIAL */
+
+  /** 持ち時間. 単位は秒 */
+  time_t total_time[2];
+
   /** 1 の場合、時間切れは負けとする */
   int flg_run_out_to_lost;
 
+  /** 消費時間の合計. それぞれの指し手ごとの消費時間は Record に記録 */
+  time_t total_elapsed_time[2];
+
+  /** 思考開始前の時刻. */
+  time_t think_start_time;
+
   /**
    * 中断等が入ったときに入力待ちループから抜けるためのフラグ。
    * 0 ならば通常通り。
@@ -358,17 +287,9 @@ struct GAME
    * 中断または投了が入った場合、入力待ちループからすぐ抜ける。
    */
   INPUTSTATUS game_status;
-  
-  /**
-   * 中断フラグへのポインタ
-   */
-  // int *chudan;
 };
 
 
-/* ------------------- function prototype ------------------- */
-
-
 ////////////////////////////////////////////////////////////////////////
 // BOARD: 局面
 
@@ -386,6 +307,10 @@ bool bo_set_by_text( BOARD* bo, const char* s);
 
 bool bo_load_file( BOARD* bo, const char* filename );
 
+std::string bo_to_fixed_str( const BOARD* bo );
+
+bool bo_set_by_fixed_str(BOARD* bo, const std::string& str);
+
 
 /** 駒種をセットする */
 static inline void boSetPiece(BOARD* bo, Xy xy, PieceKind p)
@@ -410,6 +335,7 @@ BOARD* bo_dup(const BOARD* bo);
  * @param bo 対象のBOARD
  * @param xy 座標. 1手分だけはみ出してもよい
  */
+#ifndef NDEBUG
 static inline PieceKind boGetPiece(const BOARD* bo, Xy xy)
 {
   assert(-1 <= (xy >> 4) && (xy >> 4) <= 11);
@@ -417,6 +343,9 @@ static inline PieceKind boGetPiece(const BOARD* bo, Xy xy)
 
   return bo->board[xy];
 }
+#else
+#define boGetPiece(bo, xy) ((bo)->board[xy])
+#endif // !NDEBUG
 
 
 /** sideによる攻撃 (利き) を数える */
@@ -481,9 +410,6 @@ static inline void bo_dec_repetition( BOARD* bo )
 }
 
 
-// void boSetNowTime(BOARD *bo);
-// void boSetUsedTime(BOARD *bo);
-
 /** 本将棋として局面が妥当かどうか */
 int boCheckJustGame(const BOARD* bo);
 
@@ -498,17 +424,36 @@ static inline bool operator == (const BOARD& x, const BOARD& y) {
   return !boCmp(&x, &y);
 }
 
-void printBOARD(const BOARD* bo);
+void printBOARD(const BOARD* bo, int front = 0);
 
 
 ////////////////////////////////////////////////////////////////////////
-// MOVEINFO: 候補手
+// MOVEINFO: 候補手  ohte.cc uke.cc moveinfo.cc
 
 extern const Xy arr_xy[];
 
 extern const XyDiff arr_round_to[];
 extern const XyDiff arr_round_to24[];
 
+extern const int remote_to_count[];
+extern const XyDiff remote_to[][8];
+extern const XyDiff normal_to[][8];
+extern const int normal_to_count[];
+
+
+/** 可能な合法手の最大は593 */
+#define MOVEINFOMAX 593
+
+/** 指し手 (候補手) を格納する構造体 */
+struct MOVEINFO 
+{
+  /** 指し手 */
+  TE te[MOVEINFOMAX + 1];
+  /** 指し手の数 */
+  int count;
+};
+
+
 MOVEINFO* newMOVEINFO();
 
 void freeMOVEINFO(MOVEINFO* mi);
@@ -538,6 +483,16 @@ bool mi_includes( const MOVEINFO* mi, const TE& te );
 
 void mi_append_king_moves(const BOARD* bo, MOVEINFO* mi, Xy king_xy, int next);
 
+/** 合法手をすべて生成する */
+void make_moves(const BOARD* bo, MOVEINFO* mi);
+
+void mi_append_put_moves( const BOARD* bo, Xy xy, MOVEINFO* mi );
+
+void ohte(const BOARD* bo, MOVEINFO* mi);
+
+/** 王手を受ける手を生成する */
+int uke(const BOARD* bo, MOVEINFO* mi );
+
 
 ////////////////////////////////////////////////////////////////////////
 // misc
@@ -553,18 +508,30 @@ struct RecvBuffer
   /** 行に分ける */
   typedef std::list<std::string> RecvList;
 
+  int fd;
+
   /** 行に分ける前のデータ */
   std::string sock_buffer;
 
   /** 行に分けた受信データ */
   RecvList recv_list;
 
-  int read_from_server( int server_fd );
+  RecvBuffer(int fd_): fd(fd_) { }
 
   void clear() {
+    fd = -1;
     sock_buffer = "";
     recv_list.clear();
   }
+
+  int read();
+
+  /* キューから1行取り出す */
+  std::string get_front() {
+    std::string r = recv_list.front();
+    recv_list.pop_front();
+    return r;
+  }
 };
 
 
@@ -583,6 +550,42 @@ void str_split( Container* container, const std::string& str, char delim )
   container->push_back( std::string(str, pos, i - pos) );
 }
 
+std::string str_format( const char* format, ... );
+
+void si_start_alarm();
+void si_end_alarm();
+
+
+/** 子プロセス情報 */
+struct ChildProcess 
+{
+  /** プロセスID */
+  pid_t pid;
+
+  /** 書き込み用パイプ */
+  int write_fd;
+
+  /** 読み込み用パイプ */
+  int read_fd;
+
+  /** 構造体を初期化 */
+  void clear() {
+    pid = -1; 
+    write_fd = -1; read_fd = -1;
+  }
+
+  /** 子プロセスを終了させる.
+    \todo impl.
+   */
+  void kill() {
+    assert(0); // TODO: impl
+  }
+};
+
+
+int spawn_async_with_pipe(const char* working_dir, char** argv, 
+			  ChildProcess* x);
+
 
 ////////////////////////////////////////////////////////////////////////
 // MOVEINFO2: 手順
@@ -754,11 +757,6 @@ std::string te_str( const TE& te, const BOARD& bo );
 
 void teSetTE(TE* te, Xy fm, Xy to, bool nari, PieceKind uti);
 
-int teCSAimoto(TE *te);
-int teCSAisaki(TE *te);
-int teCSAinaru(TE *te);
-void teCSASet(TE *te, int imoto, int isaki);
-
 
 static inline bool is_long_piece( PieceKind p ) __attribute__((const));
 /** 長い攻撃のできる駒か */
@@ -779,61 +777,63 @@ bool te_is_sennichite(const BOARD* bo, const TE& te, bool* forbidden);
 
 int can_move_promote(const BOARD* bo, const TE& te);
 
-
-/* moveto.cc */
-
-extern const int remote_to_count[];
-extern const XyDiff remote_to[][8];
-extern const XyDiff normal_to[][8];
-extern const int normal_to_count[];
-
-/** 合法手をすべて生成する */
-void make_moves(const BOARD* bo, MOVEINFO* mi);
-
-/** xyにある駒での指し手をmiに追加 */
-void mi_append_moveto(const BOARD* bo, Xy xy, MOVEINFO* mi);
-
-void moveto2_add(const BOARD* bo, MOVEINFO* mi, Xy xy, int next);
-
-/** 駒 pie を xy に打つ手を生成 */
-void append_a_put_move( const BOARD* bo, MOVEINFO* mi, PieceKind pie, Xy xy );
-
-void mi_append_put_moves( const BOARD* bo, Xy xy, MOVEINFO* mi );
+bool te_is_uchifuzume(const BOARD* bo, const TE& te);
 
 
-/* ohte.cc */
+void conn_send_str(int server_fd, const char* line);
+int conn_send_move(int server_fd, const BOARD* board, const DTe& te);
 
-void ohte(const BOARD* bo, MOVEINFO* mi);
-
-void normalOhte(const BOARD *bo, MOVEINFO *mi, Xy king_xy);
-// void remoteOhte(const BOARD* bo, MOVEINFO *mi, Xy king_xy);
-// void keiOhte(const BOARD* bo, MOVEINFO* mi, Xy king_xy);
-// void utiOhte(const BOARD *bo, MOVEINFO *mi, Xy king_xy);
-// void UtiOhte2(const BOARD* bo, MOVEINFO* mi, Xy king_xy);
-// void akiOhte(const BOARD* bo, MOVEINFO* mi, Xy king_xy);
+INPUTSTATUS conn_queue_front_move( std::list<std::string>& net_queue,
+				   const BOARD* bo,
+				   DTe* te, int* elapsed_time );
 
 
+////////////////////////////////////////////////////////////////////////
+// 置換表 hash.cc
 
-/* uke.c */
-
-/** 王手を受ける手を生成する */
-int uke(const BOARD* bo, MOVEINFO* mi );
-
-
-/* mate.cc */
-
-MATE_STATE dfpn_mate( BOARD* bo, int depth, TE* best_move, int* node_limit );
-
-MATE_STATE bfs_hisshi( BOARD* bo, int tesuki, int depth, TE* best_move, 
-		       int* node_limit );
-
-
-/* hash.cc */
 
 extern const uint64_t hashseed_inhand[2][8][19];
 extern const uint64_t hashval[32][16 * 10];
 struct PhiDelta;
 
+
+/** 置換表のエントリ. 持ち駒も区別する. PODでなければならない */
+struct EntVariant 
+{
+  /** 
+   * 持駒 (packed). ビットの数で表すわけではないので、
+   * 計算できるのは一致のみ. 
+   */
+  uint32_t packed_inhand[2];
+
+  /** 得点 */
+  int32_t phi, delta;
+
+  /** 記録したときの手数 (深さではない) */
+  int32_t te_count;
+
+  /** 最善手 (参考) */
+  uint32_t best_move;
+};
+// x86-64: 構造体は8バイト単位。
+#define HASH_ENT_SIZE 24
+
+
+/** 盤面を記録する置換表のエントリ. PODでなければならない */
+struct HASH
+{
+  /** 
+   * 持ち駒を含まないハッシュ値.
+   * \todo 削除する？ 
+   */
+  uint64_t key;
+
+  /** 得点. 複数記録する */
+  struct EntVariant ent[1];
+};
+#define HASH_HEADER_SIZE 8 // ent[]は除く
+
+
 /** 置換表を初期化する */
 void newHASH();
 
@@ -862,19 +862,22 @@ bool hs_minimize_proof_pieces(int offense_side, BOARD* bo);
 void hs_get_best_sequence(int level, const BOARD* bo, MOVEINFO2* mi);
 
 
-/* play */
-void play(BOARD *bo, GAME *gm);
+////////////////////////////////////////////////////////////////////////
+// 詰将棋・必至探索  mate.cc
 
-void play_move(BOARD* bo, const TE* te);
 
-void getTE2SerialString(char *buf, BOARD *bo, TE *te);
-void play_lost(BOARD *bo, GAME *gm);
-void play_toryo(BOARD *bo, GAME *gm);
-void play_win(BOARD *bo, GAME *gm);
-void play_chudan(BOARD *bo, GAME *gm);
-void play_chudan_recv(BOARD *bo, GAME *gm);
-void play_ngdata(BOARD *bo, GAME *gm);
-void putHelp(void);
+MATE_STATE dfpn_mate( BOARD* bo, int depth, TE* best_move, int* node_limit );
+
+MATE_STATE bfs_hisshi( BOARD* bo, int tesuki, int depth, TE* best_move, 
+		       int* node_limit );
+
+
+////////////////////////////////////////////////////////////////////////
+// 通常探索  minmax.cc
+
+
+INPUTSTATUS bfs_minmax(BOARD* bo, int depth, PhiDelta* score, TE* best_move);
+
 
 /* think */
 void think(BOARD *bo, MOVEINFO *mi, int depth);
@@ -884,16 +887,10 @@ void think_tsumero_uke(BOARD* bo, MOVEINFO* mi_ret);
 
 bool think_hisshi(BOARD* bo, int tenuki, int depth, TE* te_ret);
 
-void think_eval(const BOARD* bo, PhiDelta* score);
+int32_t think_eval(const BOARD* bo);
 
-int think_count_attack24( const BOARD* bo, int side, Xy target );
-int think_count_attack8( const BOARD* bo, int side, Xy target );
-
-
-/* minmax */
-
-INPUTSTATUS bfs_minmax(BOARD* bo, int depth, TE* best_move);
-int jouseki(const BOARD* bo, TE* te);
+// int think_count_attack24( const BOARD* bo, int side, Xy target );
+// int think_count_attack8( const BOARD* bo, int side, Xy target );
 
 
 /* GAME */
@@ -910,13 +907,19 @@ int gmGetComputerLevel(GAME *gm, int next);
 /** プレイヤー種別とコールバック関数を設定 */
 void gmSetInputFunc(GAME* gm, int next, InputCallbackFunc fn);
 
-#ifdef USE_SERIAL
-char *gmGetDeviceName(GAME *gm, int next);
-void gmSetDeviceName(GAME *gm, int next, char *s);
-#endif // USE_SERIAL
-
 void gmSetGameStatus(GAME* gm, INPUTSTATUS status);
 
+void gm_add_elapsed_time( GAME* gm, int side, int t );
+
+void gm_set_think_start_time(GAME* gm);
+
+extern GAME g_game;
+
+/* for GUI support */
+void si_set_pending_function( void (*f)(bool may_block) );
+
+void processing_pending( bool may_block );
+
 
 /**
  * ゲームのステータスを返す。
@@ -929,10 +932,6 @@ static inline INPUTSTATUS gmGetGameStatus(const GAME* gm)
 }
 
 
-void gmSetChudan(GAME *gm, int *chudan);
-int *gmGetChudan(GAME *gm);
-int gmGetChudanFlg(GAME *gm);
-
 /* misc */
 
 void time2string(char *buf, time_t t);
