@@ -55,7 +55,6 @@ public:
     typename Map::const_iterator it;
     if ( (it = map_.find(bo->hash_all)) != map_.end() ) {
       *v = it->second;
-      // v->phi = it->phi; v->delta = it->delta;
       return true;
     }
     return false;
@@ -63,20 +62,3 @@ public:
 };
 
 
-/** 詰め将棋での得点 */
-struct PhiDelta {
-  int32_t phi, delta;
-};
-
-static inline PhiDelta make_phi_delta(int32_t phi, int32_t delta)
-{
-  PhiDelta pd;
-  pd.phi = phi; pd.delta = delta;
-  return pd;
-}
-
-
-static inline bool operator == (const PhiDelta& x, const PhiDelta& y) 
-{
-  return x.phi == y.phi && x.delta == y.delta;
-}
