@@ -18,127 +18,142 @@
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/**
- * Sprite class
- * $Source: /cvsroot/daemonshogi/daemonshogi/src/sprite.h,v $
- * $Id: sprite.h,v 1.1.1.1 2005/12/09 09:03:07 tokita Exp $
- */
 
 #ifndef _SPRITE_H_
 #define _SPRITE_H_
 
 #include <gtk/gtk.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+// TODO: 拡張性
+#define SPRITEMAX 41
 
-  
-typedef enum {
-  KOMA_SPRITE = 0,
-  ETC_SPRITE = 1
-} SPRITE_TYPE;
 
 
 /** スプライトクラス.
  * \todo Z座標.
  * \todo 駒情報は追い出す
  */
-typedef struct {
+struct Sprite
+{
   /** スプライトの種類 */
-  SPRITE_TYPE type;
+  // SPRITE_TYPE type;
 
   /**
    * sprite の元画像. このスプライトでは画像を所有しない.
    * (src_x, src_y)-(width, height) を転送する。
    */
-  GdkPixmap* pixmap;
+  cairo_surface_t* pixmap;
 
   /** 元画像の透過マスク */
-  GdkBitmap* mask;
+  // GdkBitmap* mask;
 
-  /** sprite の表示元画像でのX位置 */
+  /** sprite の元画像でのX位置 */
   gint src_x;
-  /** sprite の表示元画像でのY位置 */
+
+  /** sprite の元画像でのY位置 */
   gint src_y;
 
-  /** sprite の横のドット数 */
-  gint width;
-  /** sprite の縦のドット数 */
-  gint height;
+  /** sprite の元画像での横のドット数 */
+  int src_width;
+
+  /** sprite の元画像での縦のドット数 */
+  int src_height;
   
-  /** sprite のX位置 */	
+  /** sprite の表示先でのX位置 */	
   gint x;
-  /** sprite のY位置 */	
+  /** sprite の表示先でのY位置 */	
   gint y;
+
+  /** 表示の際に大きさを拡大する。座標はスケールさせない。 */
+  double scale_size_x, scale_size_y;
+
   /** ドラッグ前のX位置 */
   gint from_x;
   /** ドラッグ前のY位置 */
   gint from_y;
+
   /** 表示するか？のフラグ。TRUEならば表示する。 */
   gboolean visible;
+
   /** 移動できるかのフラグ。TRUEならば移動できる。 */
-  gboolean move;
-  /** 駒種類 */
-  gint koma;
-} Sprite;
+  gboolean movable;
+
+  /** ユーザデータ */
+  void* userdata;
+};
+
 
 /*********************************************************/
 /* function prototypes */
 /*********************************************************/
 
-Sprite*    daemon_sprite_new( GdkPixmap* pixmap,
-                              GdkBitmap* bitmap,
+Sprite*    daemon_sprite_new( cairo_surface_t* pixmap,
+                              // GdkBitmap* bitmap,
                               int src_x, int src_y,
                               gint width,
                               gint height);
 
-SPRITE_TYPE daemon_sprite_get_type  (Sprite* sprite);
+void daemon_sprite_draw_all( Sprite* sprites[], cairo_surface_t* drawable );
 
-gint       daemon_sprite_get_width  (const Sprite* sprite);
-gint       daemon_sprite_get_height (const Sprite* sprite);
+void daemon_sprite_draw_overlap( Sprite* sprites[], const GdkRectangle* rect,
+				 cairo_surface_t* drawable );
+
+// SPRITE_TYPE daemon_sprite_get_type  (Sprite* sprite);
+
+int daemon_sprite_get_width( const Sprite* sprite );
+int daemon_sprite_get_height( const Sprite* sprite );
 
 gint       daemon_sprite_get_x      (const Sprite* sprite);
 gint       daemon_sprite_get_y      (const Sprite* sprite);
 
-// gint       daemon_sprite_get_src_x  (Sprite* sprite);
-// gint       daemon_sprite_get_src_y  (Sprite* sprite);
+int daemon_sprite_get_src_x( const Sprite* sprite );
+int daemon_sprite_get_src_y( const Sprite* sprite );
 
-void daemon_sprite_set_pixmap( Sprite* sprite, GdkPixmap *pixmap, GdkBitmap* mask,
+void daemon_sprite_set_src_xy( Sprite* sprite, int src_x, int src_y );
+
+void daemon_sprite_set_pixmap( Sprite* sprite, cairo_surface_t* pixmap, 
+			       // GdkBitmap* mask,
                                int src_x, int src_y, int width, int height );
 
-GdkPixmap* daemon_sprite_get_pixmap (Sprite* sprite);
-gboolean   daemon_sprite_isvisible  (Sprite* sprite);
-void       daemon_sprite_set_type   (Sprite* sprite,
-				     SPRITE_TYPE type);
+cairo_surface_t* daemon_sprite_get_pixmap( Sprite* sprite );
+
+gboolean daemon_sprite_isvisible( const Sprite* sprite );
+
+// void       daemon_sprite_set_type   (Sprite* sprite,
+// 				     SPRITE_TYPE type);
 
 void       daemon_sprite_set_xy( Sprite* sprite, int x, int y );
 
 gboolean   daemon_sprite_on         (Sprite* sprite,
 				     int x,
 				     int y);
-void       daemon_sprite_set_mask   (Sprite* sprite,
-				     GdkBitmap* mask);
-GdkBitmap* daemon_sprite_get_mask   (Sprite* sprite);
+
+//void       daemon_sprite_set_mask   (Sprite* sprite,
+//				     GdkBitmap* mask);
+
+// GdkBitmap* daemon_sprite_get_mask   (Sprite* sprite);
+
 void       daemon_sprite_set_visible(Sprite* sprite,
 				     gboolean bool_);
-void       daemon_sprite_set_koma   (Sprite* sprite,
-				     gint koma);
-gint       daemon_sprite_get_koma   (Sprite* sprite);
-gboolean   daemon_sprite_ismove     (Sprite* sprite);
-void       daemon_sprite_set_move   (Sprite* sprite, gboolean move);
-void       daemon_sprite_set_from_x (Sprite* sprite,
-				     gint x);
-void       daemon_sprite_set_from_y (Sprite* sprite,
-				     gint y);
+
+// void       daemon_sprite_set_koma   (Sprite* sprite,
+// 				     gint koma);
+
+void* daemon_sprite_get_userdata( Sprite* sprite );
+
+gboolean daemon_sprite_ismovable( const Sprite* sprite );
+
+void       daemon_sprite_set_movable( Sprite* sprite, gboolean move);
+
+void daemon_sprite_set_drag_from( Sprite* sprite, int x, int y );
+
 gint       daemon_sprite_get_from_x (Sprite* sprite);
 gint       daemon_sprite_get_from_y (Sprite* sprite);
 
-void daemon_sprite_draw(Sprite* sprite, GdkPixmap* drawable, GdkGC* gc);
+void daemon_sprite_draw( Sprite* sprite, cairo_surface_t* drawable );
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+int daemon_sprite_point_on_sprite( Sprite* sprites[], int pt_x, int pt_y );
 
+void daemon_sprite_set_scale( Sprite* sprite, double xscale, double yscale );
 
 #endif /* _SPRITE_H_ */
