@@ -38,9 +38,7 @@ class BoardTest: public TestCase
   CPPUNIT_TEST(test_3);
   CPPUNIT_TEST(test_4);
   CPPUNIT_TEST(test_4a);
-#ifdef ENABLE_SPEED_TEST
-  CPPUNIT_TEST(test_5);
-#endif // ENABLE_SPEED_TEST
+  CPPUNIT_TEST(test_6);
   CPPUNIT_TEST_SUITE_END();
 
   BOARD* bo;
@@ -224,34 +222,35 @@ public:
     CPPUNIT_ASSERT( bo->key == bo->hash_all );
   }
 
-#ifdef ENABLE_SPEED_TEST
-  void sub5(int depth, BOARD* bo)
+  /** 固定長文字列のテスト */
+  void test_6()
   {
-    if (depth <= 0)
-      return;
+    CPPUNIT_ASSERT_EQUAL(
+string("IJKLOLKJI/0M00000N0/HHHHHHHHH/000000000/000000000/000000000/111111111/070000060/234585432aaaaaaaaaaaaaa+"), 
+bo_to_fixed_str(bo) );
+    
+    // http://www.hakusa.net/shogi/column/58.html
+    bo_set_by_text( bo,
+		    "P1+UM-KE-GI *  *  *  * -KE-KY\n"
+		    "P2 *  *  *  *  *  * -HI *  * \n"
+		    "P3-FU-FU * -OU-FU *  *  *  * \n"
+		    "P4 *  * -FU * -KI *  *  * -FU\n"
+		    "P5+FU * +FU *  *  *  *  *  * \n"
+		    "P6 *  *  *  *  * -FU * +FU * \n"
+		    "P7 * -GI *  * -GI * +FU * +FU\n"
+		    "P8 *  *  *  *  * +HI+KI * +KY\n"
+		    "P9+KY+KE * -TO *  *  * +KE+OU\n"
+		    "P+00KI00GI00KY00FU00FU\n"
+		    "P-00KA00KI00FU00FU00FU00FU\n"
+		    "+\n" );
+    uint64_t hash = bo->hash_all;
+    string s = bo_to_fixed_str(bo);
 
-    MOVEINFO mi;
-    make_moves(bo, &mi);
-    for (int j = 0; j < mi.count; j++) {
-#if 1 // 2009.9.24現在 こちらが速い
-      boMove_mate(bo, mi.te[j]);
-      sub5(depth - 1, bo);
-      boBack_mate(bo);
-#else
-      BOARD* bo2 = bo_dup(bo);
-      boMove_mate(bo2, mi.te[j]);
-      sub5(depth - 1, bo2);
-      freeBOARD(bo2);
-#endif
-    }
+    bo_reset_to_hirate(bo);
+    bo_set_by_fixed_str(bo, s);
+    CPPUNIT_ASSERT( hash == bo->hash_all );
   }
 
-  /** 局面を戻すのとコピーするのはどちらが速いか */
-  void test_5() 
-  {
-    sub5(5, bo);
-  }
-#endif // ENABLE_SPEED_TEST
 };
 CPPUNIT_TEST_SUITE_REGISTRATION( BoardTest );
 
