@@ -22,6 +22,7 @@
 #include <memory>
 #include <stdio.h>
 #include <si/si.h>
+#include <si/si-think.h>
 #include "tests.h"
 #include <sys/time.h>
 
@@ -47,10 +48,6 @@ class GenMovesTest: public TestCase
   CPPUNIT_TEST( test_7 );
   CPPUNIT_TEST( test_8 );
   CPPUNIT_TEST( test_9 );
-
-  // 速度テスト
-  CPPUNIT_TEST( test_10 );
-
   CPPUNIT_TEST_SUITE_END();
 
   BOARD* bo;
@@ -409,11 +406,11 @@ public:
     ohte( bo, &mi );
     printf("%s: %d\n", __func__, __LINE__);
     mi_print(&mi, *bo);
-    CPPUNIT_ASSERT_EQUAL(2 + 1, mi.count); // 打ち歩詰めも生成される
+    CPPUNIT_ASSERT_EQUAL(2, mi.count); // 打ち歩詰めは生成しない
 
-    // 合法手の生成でも打ち歩詰めの手は生成される
+    // 合法手の生成でも打ち歩詰めの手は生成しない
     make_moves(bo, &mi);
-    CPPUNIT_ASSERT_EQUAL( 7 + 16 + 14 + (72 - 10 - 1) + 1, mi.count );
+    CPPUNIT_ASSERT_EQUAL( 7 + 16 + 14 + (72 - 10 - 1), mi.count );
 
     // 手は妥当ではない
     CPPUNIT_ASSERT_EQUAL( false, te_is_valid(bo, te_make_put(0x65, FU)) );
@@ -426,7 +423,8 @@ public:
     CPPUNIT_ASSERT( dfpn_mate(bo, 3, &te_ret, &node_limit) != CHECK_MATE );
 
     // 通常探索でも同じ
-    bfs_minmax(bo, 3, &te_ret);
+    PhiDelta score;
+    bfs_minmax(bo, 3, &score, &te_ret);
     CPPUNIT_ASSERT( teCmpTE(te_ret, te_make_put(0x65, FU)) );
   }
 
@@ -514,39 +512,5 @@ public:
     CPPUNIT_ASSERT_EQUAL(4, mi.count);
   }
 
-  // http://d.hatena.ne.jp/ak11/20091107#p1
-  void test_10()
-  {
-    struct timeval tv1, tv2;
-    gettimeofday( &tv1, NULL );
-
-    bo_set_by_text( bo,
-"P1-KY *  *  *  *  *  * -KE-KY\n"
-"P2 *  *  *  *  * +TO * -KI-OU\n"
-"P3 *  * -KE-FU * +GI *  *  * \n"
-"P4-FU * -FU *  *  *  * +FU-FU\n"
-"P5 *  *  * +FU *  * +GI-FU * \n"
-"P6 * +FU+FU-KA *  * +FU * +FU\n"
-"P7+FU *  *  *  *  * +KI+GI * \n"
-"P8+HI *  *  *  *  *  *  *  * \n"
-"P9+KY+KE *  *  *  * -KA+OU+KY\n"
-"P+00HI00KI\n"
-"P-00KI00GI00KE\n"
-"P-00FU00FU00FU00FU00FU\n"
-"-\n" );
-    int COUNT = 1000000 * 3;
-    MOVEINFO mi;
-    for (int i = 0; i < COUNT; i++) {
-      make_moves(bo, &mi);
-      // miRemoveDuplicationTE(&mi);
-    }
-
-    gettimeofday( &tv2, NULL );
-
-    CPPUNIT_ASSERT_EQUAL( 207, mi.count ); // 角の不成もカウント
-    printf("time = (%ld, %ld)\n", tv2.tv_sec - tv1.tv_sec, 
-	   tv2.tv_usec - tv1.tv_usec );
-  }
 };
 CPPUNIT_TEST_SUITE_REGISTRATION( GenMovesTest );
-
